/*
  Copyright (C) 1998, 1999 Gerd Rausch <gerd@alf.gun.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdarg.h>
#include <asm/errno.h>

#include "i2c.h"

#define I2C_MAX_POLL	0x1000

int samI2CWrite(SamCard *card,
	     int (*output)(SamCard *card, int sda, int scl),
	     int (*input)(SamCard *card, int *sda, int *scl),
	     int n, ...)
{
  va_list ap;
  int result, sda, scl, v, i, j;

  /* send START */
  if((result=(*output)(card, 1, 1))<0 ||
     (result=(*output)(card, 0, 1))<0 ||
     (result=(*output)(card, 0, 0))<0) {
    (*output)(card, 1, 1);
    return result;
  }

  va_start(ap, n);
  for(i=0; result>=0 && i<n; i++) {
    v=va_arg(ap, int);
    for(j=0; j<8; j++) {
      sda=!!(v&0x80);
      if((result=(*output)(card, sda, 0))<0 ||
	 (result=(*output)(card, sda, 1))<0 ||
	 (result=(*output)(card, sda, 0))<0)
	break;
      v<<=1;
    }
    if(result<0)
      break;
    if((result=(*output)(card, 1, 0))<0 ||
       (result=(*output)(card, 1, 1))<0)
      break;
    sda=scl=1;
    for(j=0; result>=0 && j<I2C_MAX_POLL && (sda || !scl); j++)
      result=(*input)(card, &sda, &scl);
    if(result<0 ||
       (result=(*output)(card, 1, 0))<0)
      break;
    if(j>=I2C_MAX_POLL)
      result=-EIO;
  }
  va_end(ap);

  if(result<0) {
    (*output)(card, 1, 1);
    return result;
  }

  /* send STOP */
  if((result=(*output)(card, 0, 1))<0 ||
     (result=(*output)(card, 1, 1))<0) {
    (*output)(card, 1, 1);
    return result;
  }
  
  return 0;
}
