#ifndef _SAM9407_H
#define _SAM9407_H

/* SAM9407 specific ioctl's */

#include <linux/ioctl.h>

#define SAM_NROF_SBANKS		8
#define SAM_SBANK_NAME_LEN	8

/***** /dev/audio ioctl's *****/

typedef struct {
  unsigned short left, right, auxLeft, auxRight;
} SamVolume;

#define SAM_IOC_VOLUME_GET	_IOR('S', 0, SamVolume)

#define SAM_IOC_VOLUME_SET	_IOW('S', 1, SamVolume)

/***** /dev/mixer ioctl's *****/

/* notifications (bytes) that are read(2) from /dev/mixer
 * are of type SamMixerId if >=0 or one of the following:
 */
#define SAM_MIXER_FIRMWARE_CHANGED	-1
#define SAM_MIXER_SBANK_CHANGED		-2

typedef struct {
  char hardware[16];
  char firmware[16];
  unsigned memTotal, memFree, memSBanks, memAudio;
  unsigned playChannelsUsed, playChannelsTotal;
  unsigned recordChannelsUsed, recordChannelsTotal;
} SamDriverInfo;

typedef struct {
  unsigned short *data;
  unsigned size;
} SamFirmware;

/* control id's for SamMixerValue */
typedef enum {
  SAM_MIXER_ID_SENTINEL,	/* used to mark end of vector */
  SAM_MIXER_ID_MASTER_VOL,
  SAM_MIXER_ID_REC_MODE,
  SAM_MIXER_ID_REC_VOL,
  SAM_MIXER_ID_EQ_LBL,
  SAM_MIXER_ID_EQ_MLBL,
  SAM_MIXER_ID_EQ_MHBL,
  SAM_MIXER_ID_EQ_HBL,
  SAM_MIXER_ID_EQ_LBR,
  SAM_MIXER_ID_EQ_MLBR,
  SAM_MIXER_ID_EQ_MHBR,
  SAM_MIXER_ID_EQ_HBR,
  SAM_MIXER_ID_EQF_LB,
  SAM_MIXER_ID_EQF_MLB,
  SAM_MIXER_ID_EQF_MHB,
  SAM_MIXER_ID_EQF_HB,
  SAM_MIXER_ID_AUD_SEL,
  SAM_MIXER_ID_AUD_GAINL,
  SAM_MIXER_ID_AUD_GAINR,
  SAM_MIXER_ID_AUDCHR_SEND,
  SAM_MIXER_ID_GMREV_SEND,
  SAM_MIXER_ID_GMCHR_SEND,
  SAM_MIXER_ID_AUDREV_SEND,
  SAM_MIXER_ID_ECH_LEV,
  SAM_MIXER_ID_ECH_TIM,
  SAM_MIXER_ID_ECH_FEED,
  SAM_MIXER_ID_SUR_VOL,
  SAM_MIXER_ID_SUR_DEL,
  SAM_MIXER_ID_SUR_INP,
  SAM_MIXER_ID_SUR_24,
  SAM_MIXER_ID_AUDL_VOL,
  SAM_MIXER_ID_AUDR_VOL,
  SAM_MIXER_ID_AUDL_PAN,
  SAM_MIXER_ID_AUDR_PAN,
  SAM_MIXER_ID_GM_VOL,
  SAM_MIXER_ID_GM_PAN,
  SAM_MIXER_ID_REV_VOL,
  SAM_MIXER_ID_CHR_VOL,
  SAM_MIXER_ID_GM_POST,
  SAM_MIXER_ID_WAVE_POST,
  SAM_MIXER_ID_MOD_POST,
  SAM_MIXER_ID_AUDECH_POST,
  SAM_MIXER_ID_EFF_POST,
  SAM_MIXER_ID_REV_TYPE,
  SAM_MIXER_ID_CHR_TYPE,
  SAM_MIXER_ID_CHR_DEL,
  SAM_MIXER_ID_CHR_FEED,
  SAM_MIXER_ID_CHR_RATE,
  SAM_MIXER_ID_CHR_DEPTH,
  SAM_MIXER_ID_REV_TIME,
  SAM_MIXER_ID_REV_FEED,
  /* config messages: those will reset the firmware
   * and can be used only if sam9407 is not busy
   */
  SAM_MIXER_ID_REC_CFG,
  SAM_MIXER_ID_AUD_ONOFF,
  SAM_MIXER_ID_ECH_ONOFF,
  SAM_MIXER_ID_REV_ONOFF,
  SAM_MIXER_ID_CHR_ONOFF,
  SAM_MIXER_ID_EQU_TYPE,
  SAM_MIXER_ID_SUR_ONOFF,
  SAM_MIXER_ID_WAVE_ASS,
  SAM_MIXER_ID_MOD_ASS,

  SAM_MIXER_ID_WAV_VOL0,
  SAM_MIXER_ID_WAV_VOL1,
  SAM_MIXER_ID_WAV_VOL2,
  SAM_MIXER_ID_WAV_VOL3,
  SAM_MIXER_ID_WAV_VOL4,
  SAM_MIXER_ID_WAV_VOL5,
  SAM_MIXER_ID_WAV_VOL6,
  SAM_MIXER_ID_WAV_VOL7,
  SAM_MIXER_ID_WAV_VOL8,

  SAM_MIXER_ID_WAV_AUX_VOL0,
  SAM_MIXER_ID_WAV_AUX_VOL1,
  SAM_MIXER_ID_WAV_AUX_VOL2,
  SAM_MIXER_ID_WAV_AUX_VOL3,
  SAM_MIXER_ID_WAV_AUX_VOL4,
  SAM_MIXER_ID_WAV_AUX_VOL5,
  SAM_MIXER_ID_WAV_AUX_VOL6,
  SAM_MIXER_ID_WAV_AUX_VOL7,
  SAM_MIXER_ID_WAV_AUX_VOL8,

  SB_MIXER_ID_MASTER,
  SB_MIXER_ID_WAV,
  SB_MIXER_ID_FM,
  SB_MIXER_ID_CD,
  SB_MIXER_ID_LINE,
  SB_MIXER_ID_MIC,

  SAM_MIXER_ID_LAST
} SamMixerId;

typedef struct {
  SamMixerId id;
  unsigned value;
} SamMixerValue;

typedef struct {
  unsigned char id;
  char *data;
  unsigned size;
} SamSoundBank;

typedef struct {
  unsigned char id;
  unsigned short priority;
} SamSoundBankPrio;

typedef struct {
  char name[SAM_SBANK_NAME_LEN+1];
  unsigned short priority;
} SamSoundBankInfo;

/* retrieve driver information  */
#define SAM_IOC_DRIVER_INFO	_IOR('S', 0, SamDriverInfo)

/* upload firmware */
#define SAM_IOC_FIRMWARE_LOAD	_IOW('S', 1, SamFirmware)

/* retrieve mixer values */
#define SAM_IOC_MIXER_GET	_IOWR('S', 2, SamMixerValue)

/* set mixer values */
#define SAM_IOC_MIXER_SET	_IOW('S', 3, SamMixerValue)

/* retrieve supported mixer options */
#define SAM_IOC_MIXER_INFO	_IOWR('S', 4, SamMixerValue)

/* upload a .94B file */
#define SAM_IOC_BANK_LOAD	_IOW('S', 5, SamSoundBank)

/* dispose a soundbank */
#define SAM_IOC_BANK_UNLOAD	_IOW('S', 6, unsigned char)

/* change soundbank priority */
#define SAM_IOC_BANK_PRIO_CHG	_IOW('S', 7, SamSoundBankPrio)

/* query soundbanks */
#define SAM_IOC_BANK_INFO	_IOR('S', 8, SamSoundBankInfo)

//+kjk
/* set daad */
#define SAM_IOC_SET_DAAD	_IOW('S', 9, unsigned short)

#endif /* _SAM9407_H */
