/*
  Copyright (C) 1998, 1999 Gerd Rausch <gerd@alf.gun.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/version.h>
#if LINUX_VERSION_CODE<0x20200
#include <linux/bios32.h>
#endif
#include <linux/pci.h>
#include <asm/io.h>

#include "driver.h"
#include "io.h"
#include "i2c.h"

/* indexed by SamMixerId */
static SamMixerVendorInfo samSoundTrackMixerInfo[]={
  {				/* SAM_MIXER_ID_SENTINEL */
  },
  {				/* SAM_MIXER_ID_MASTER_VOL */
    SamMixerTypeByte,
    0x07,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REC_MODE */
    SamMixerTypeBoolean,
    0x08,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REC_VOL */
    SamMixerTypeByte,
    0x0A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_LBL */
    SamMixerType7Bit,
    0x10,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MLBL */
    SamMixerType7Bit,
    0x11,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MHBL */
    SamMixerType7Bit,
    0x12,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_HBL */
    SamMixerType7Bit,
    0x13,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_LBR */
    SamMixerType7Bit,
    0x14,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MLBR */
    SamMixerType7Bit,
    0x15,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MHBR */
    SamMixerType7Bit,
    0x16,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_HBR */
    SamMixerType7Bit,
    0x17,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_LB */
    SamMixerType7Bit,
    0x18,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_MLB */
    SamMixerType7Bit,
    0x19,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_MHB */
    SamMixerType7Bit,
    0x1A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_HB */
    SamMixerType7Bit,
    0x1B,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUD_SEL */
    SamMixerTypeBoolean,
    0x20,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUD_GAINL */
    SamMixerType7Bit,
    0x21,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUD_GAINR */
    SamMixerType7Bit,
    0x22,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDCHR_SEND */
    SamMixerType7Bit,
    0x24,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GMREV_SEND */
    SamMixerTypeByte,
    0x25,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GMCHR_SEND */
    SamMixerTypeByte,
    0x26,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDREV_SEND */
    SamMixerType7Bit,
    0x27,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_ECH_LEV */
    SamMixerType7Bit,
    0x28,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_ECH_TIM */
    SamMixerType7Bit,
    0x29,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_ECH_FEED */
    SamMixerType7Bit,
    0x2A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_VOL */
    SamMixerTypeByte,
    0x30,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_DEL */
    SamMixerType7Bit,
    0x31,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_INP */
    SamMixerTypeBoolean,
    0x32,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_24 */
    SamMixerTypeBoolean,
    0x33,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDL_VOL */
    SamMixerTypeByte,
    0x34,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDR_VOL */
    SamMixerTypeByte,
    0x35,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDL_PAN */
    SamMixerType7Bit,
    0x36,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDR_PAN */
    SamMixerType7Bit,
    0x37,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GM_VOL */
    SamMixerTypeByte,
    0x38,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GM_PAN */
    SamMixerType7Bit,
    0x39,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_VOL */
    SamMixerTypeByte,
    0x3A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_VOL */
    SamMixerTypeByte,
    0x3B,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GM_POST */
    SamMixerTypeBoolean,
    0x62,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAVE_POST */
    SamMixerTypeBoolean,
    0x63,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_MOD_POST */
    SamMixerTypeBoolean,
    0x64,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDECH_POST */
    SamMixerTypeBoolean,
    0x65,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EFF_POST */
    SamMixerTypeBoolean,
    0x66,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_TYPE */
    SamMixerType3Bit,
    0x69,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_TYPE */
    SamMixerType3Bit,
    0x6A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_DEL */
    SamMixerType7Bit,
    0x74,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_FEED */
    SamMixerType7Bit,
    0x75,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_RATE */
    SamMixerType7Bit,
    0x76,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_DEPTH */
    SamMixerType7Bit,
    0x77,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_TIME */
    SamMixerType7Bit,
    0x78,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_FEED */
    SamMixerType7Bit,
    0x79,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REC_CFG */
    SamMixerTypeBoolean,
    0x09,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_AUD_ONOFF */
    SamMixerTypeBoolean,
    0x6F,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_ECH_ONOFF */
    SamMixerTypeBoolean,
    0x68,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_REV_ONOFF */
    SamMixerTypeBoolean,
    0x6C,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_CHR_ONOFF */
    SamMixerTypeBoolean,
    0x6D,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_EQU_TYPE */
    SamMixerType0To2,
    0x6B,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_SUR_ONOFF */
    SamMixerTypeBoolean,
    0x6E,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_WAVE_ASS */
    SamMixerTypeBoolean,
    0x60,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_MOD_ASS */
    SamMixerTypeBoolean,
    0x61,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_WAV_VOL0 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL1 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL2 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL3 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL4 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL5 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL6 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL7 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_VOL8 */
    SamMixerTypeWav,
    0x45,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL0 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL1 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL2 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL3 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL4 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL5 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL6 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL7 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAV_AUX_VOL8 */
    SamMixerTypeWav,
    0x47,
    -1,				/* storeAt */
  },

  {				/* SB_MIXER_ID_MASTER */
    SamMixerTypeSB,
    0x22,
    -1,				/* storeAt */
  },
  {				/* SB_MIXER_ID_WAV */
    SamMixerTypeSB,
    0x4,
    -1,				/* storeAt */
  },
  {				/* SB_MIXER_ID_FM */
    SamMixerTypeSB,
    0x26,
    -1,				/* storeAt */
  },
  {				/* SB_MIXER_ID_CD */
    SamMixerTypeSB,
    0x28,
    -1,				/* storeAt */
  },
  {				/* SB_MIXER_ID_LINE */
    SamMixerTypeSB,
    0x2e,
    -1,				/* storeAt */
  },
  {				/* SB_MIXER_ID_MIC */
    SamMixerTypeSB,
    0xa,
    -1,				/* storeAt */
  },

};

static SamMixerVendorInfo samEWS64MixerInfo[]={
  {				/* SAM_MIXER_ID_SENTINEL */
  },
  {				/* SAM_MIXER_ID_MASTER_VOL */
    SamMixerTypeByteEWS64,
    0x20,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REC_MODE */
    SamMixerTypeBoolean,
    0x08,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REC_VOL */
    SamMixerTypeByteEWS64Twice,
    0x0A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_LBL */
    SamMixerType7Bit,
    0x10,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MLBL */
    SamMixerType7Bit,
    0x11,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MHBL */
    SamMixerType7Bit,
    0x12,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_HBL */
    SamMixerType7Bit,
    0x13,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_LBR */
    SamMixerType7Bit,
    0x14,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MLBR */
    SamMixerType7Bit,
    0x15,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_MHBR */
    SamMixerType7Bit,
    0x16,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQ_HBR */
    SamMixerType7Bit,
    0x17,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_LB */
    SamMixerType7Bit,
    0x18,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_MLB */
    SamMixerType7Bit,
    0x19,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_MHB */
    SamMixerType7Bit,
    0x1A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EQF_HB */
    SamMixerType7Bit,
    0x1B,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUD_SEL */
    SamMixerTypeUnsupported
  },
  {				/* SAM_MIXER_ID_AUD_GAINL */
    SamMixerTypeUnsupported
  },
  {				/* SAM_MIXER_ID_AUD_GAINR */
    SamMixerTypeUnsupported
  },
  {				/* SAM_MIXER_ID_AUDCHR_SEND */
    SamMixerType7Bit,
    0x24,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GMREV_SEND */
    SamMixerTypeByte,
    0x25,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GMCHR_SEND */
    SamMixerTypeByte,
    0x26,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDREV_SEND */
    SamMixerType7Bit,
    0x27,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_ECH_LEV */
    SamMixerType7Bit,
    0x28,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_ECH_TIM */
    SamMixerType7Bit,
    0x29,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_ECH_FEED */
    SamMixerType7Bit,
    0x2A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_VOL */
    SamMixerTypeByteEWS647F,
    0x30,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_DEL */
    SamMixerType7Bit,
    0x31,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_INP */
    SamMixerTypeBoolean,
    0x32,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_SUR_24 */
    SamMixerTypeBoolean,
    0x33,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDL_VOL */
    SamMixerTypeByteEWS64,
    0x34,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDR_VOL */
    SamMixerTypeByteEWS64,
    0x35,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDL_PAN */
    SamMixerTypeUnsupported
  },
  {				/* SAM_MIXER_ID_AUDR_PAN */
    SamMixerTypeUnsupported
  },
  {				/* SAM_MIXER_ID_GM_VOL */
    SamMixerTypeByteEWS647F,
    0x38,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GM_PAN */
    SamMixerType7Bit,
    0x39,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_VOL */
    SamMixerTypeByteEWS647F,
    0x3A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_VOL */
    SamMixerTypeByteEWS647F,
    0x3B,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_GM_POST */
    SamMixerTypeBoolean,
    0x62,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_WAVE_POST */
    SamMixerTypeBoolean,
    0x63,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_MOD_POST */
    SamMixerTypeBoolean,
    0x64,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_AUDECH_POST */
    SamMixerTypeBoolean,
    0x65,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_EFF_POST */
    SamMixerTypeBoolean,
    0x66,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_TYPE */
    SamMixerType3Bit,
    0x69,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_TYPE */
    SamMixerType3Bit,
    0x6A,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_DEL */
    SamMixerType7Bit,
    0x74,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_FEED */
    SamMixerType7Bit,
    0x75,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_RATE */
    SamMixerType7Bit,
    0x76,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_CHR_DEPTH */
    SamMixerType7Bit,
    0x77,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_TIME */
    SamMixerType7Bit,
    0x78,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REV_FEED */
    SamMixerType7Bit,
    0x79,
    -1,				/* storeAt */
  },
  {				/* SAM_MIXER_ID_REC_CFG */
    SamMixerTypeUnsupported
  },
  {				/* SAM_MIXER_ID_AUD_ONOFF */
    SamMixerTypeBoolean,
    0x6F,
    SAM_OFFSET_OF(SamEWS64CtrlValues, aud_onoff),	/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_ECH_ONOFF */
    SamMixerTypeBoolean,
    0x68,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_REV_ONOFF */
    SamMixerTypeBoolean,
    0x6C,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_CHR_ONOFF */
    SamMixerTypeBoolean,
    0x6D,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_EQU_TYPE */
    SamMixerType0To2,
    0x6B,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_SUR_ONOFF */
    SamMixerTypeBoolean,
    0x6E,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_WAVE_ASS */
    SamMixerTypeBoolean,
    0x60,
    -1,				/* storeAt */
    1				/* doesReset */
  },
  {				/* SAM_MIXER_ID_MOD_ASS */
    SamMixerTypeBoolean,
    0x61,
    -1,				/* storeAt */
    1				/* doesReset */
  },
};

static int ews64Setup(SamCard *card)
{
  /* change samplerate */
  unsigned sampleRate=44100;
  return samTalk(card,
		 SamSendCtrl, 3, 0x07, sampleRate>>8, sampleRate,
		 SamEndOfScript);
}

/* indexed by FirmwareType */
SamFirmwareInfo samFirmwareInfos[]={
  {				/* SamFirmwareTypeGenuine */
    "genuine",			/* name */
    0,				/* probeAddr */
    NULL,			/* probeStr */
    0,				/* verifyChecksum */
    NULL,			/* setup */
    64,				/* mmtSlots */
    8,				/* playBackChannels */
    1,				/* recordChannels */
    0x00,			/* getMMTAck */
    0x00,			/* openAck */
    0xC0,			/* closeAck */
    0,				/* closeAckRecChannel */
    0,				/* volFormatEWS64 */
    samSoundTrackMixerInfo,	/* mixerInfo */
    0x00,			/* resetHookCtrl */
    0,				/* nResetReadjust */
    NULL,			/* resetReadjust */
    0,				/* nSBankRehash */
    NULL,			/* sbankRehash */
    0,				/* ctrlValuesSize */
  },
  {				/* SamFirmwareTypeEWS64 */
    "ews64",			/* name */
    0x406,			/* probeAddr */
    "EWS",			/* probeStr */
    1,				/* verifyChecksum */
    ews64Setup,			/* setup */
    256,			/* mmtSlots */
    32,				/* playBackChannels */
    1,				/* recordChannels */
    0x01,			/* getMMTAck */
    0xBD,			/* openAck */
    0xDE,			/* closeAck */
    1,				/* closeAckRecChannel */
    1,				/* volFormatEWS64 */
    samEWS64MixerInfo,		/* mixerInfo */
    0x23,			/* resetHookCtrl */
    2,				/* nResetReadjust */
    "\x39\x40",			/* resetReadjust */
    2,				/* nSBankRehash */
    "\x7E\x00",			/* sbankRehash */
    sizeof(SamEWS64CtrlValues),	/* ctrlValuesSize */
  },
};

static int ews64CtrlOut(SamCard *card, int sda, int scl)
{
  udelay(200);
  outb(0xF9|(sda<<2)|(scl<<1), card->auxPort0);
  return 0;
}

static int ews64CtrlIn(SamCard *card, int *sda, int *scl)
{
  int v;
  udelay(200);
  v=inb(card->auxPort0);
  *sda=(v>>2)&1;
  *scl=(v>>1)&1;
  return 0;
}

static int ews64sReset(SamCard *card)
{
  int error;
  if((error=samI2CWrite(card, ews64CtrlOut, ews64CtrlIn, 2, 0x40, 0xDE))<0 ||
     (error=samI2CWrite(card, ews64CtrlOut, ews64CtrlIn, 2, 0x40, 0x5E))<0) {
    printk(KERN_ERR "sam9407: EWS64 digital controller is not responding\n");
    return error;
  }

  return 0;
}

static int ews64xlReset(SamCard *card)
{
  int error;
  if((error=samI2CWrite(card, ews64CtrlOut, ews64CtrlIn, 2, 0x40, 0xDE))<0 ||
     (error=samI2CWrite(card, ews64CtrlOut, ews64CtrlIn, 3, 0x40, 0x5E, 0x08))<0) {
    printk(KERN_ERR "sam9407: EWS64 digital controller is not responding\n");
    return error;
  }

  return 0;
}

SamHardwareInfo samHardwareInfos[]={
  {				/* SamHardwareTypeUnknown */
    "unknown",			/* name */
    NULL,			/* reset */
  },
  {				/* SamHardwareTypeEWS64S */
    "ews64s",			/* name */
    ews64sReset,		/* reset */
  },
  {				/* SamHardwareTypeEWS64XL */
    "ews64xl",			/* name */
    ews64xlReset,		/* reset */
  },
  {				/* SamHardwareTypeST128 */
    "ST97PNP",			/* name */
    NULL,			/* reset */
  },
  {				/* SamHardwareTypeST128 */
    "ST128DDMA",			/* name */
    NULL,			/* reset */
  },
  {				/* SamHardwareTypeMAXI64 */
    "maxi64",			/* name */
    NULL,			/* reset */
  },
};

static SamPnpResource ews64sPnpResources[]={
  {				/* SamHardwareResPort */
    0,				/* isOptional */
    "sam9407 (EWS64S)",
    4, 0
  },
  {				/* SamHardwareResIrq */
    1,				/* isOptional */
    "sam9407 (EWS64S)",
    4, 0
  },
  {				/* SamHardwareResAuxPort0 */
    0,				/* isOptional */
    "sam9407 (EWS64S Dig. Ctrl #0)",
    3, 0
  },
  {				/* SamHardwareResAuxPort1 */
    1,				/* isOptional */
    "sam9407 (EWS64S Dig. Ctrl #1)",
    3, 1
  },
};

static SamPnpResource ews64xlPnpResources[]={
  {				/* SamHardwareResPort */
    0,				/* isOptional */
    "sam9407 (EWS64XL)",
    4, 0
  },
  {				/* SamHardwareResIrq */
    1,				/* isOptional */
    "sam9407 (EWS64XL)",
    4, 0
  },
  {				/* SamHardwareResAuxPort0 */
    0,				/* isOptional */
    "sam9407 (EWS64XL Dig. Ctrl)",
    5, 0
  },
  {				/* SamHardwareResAuxPort1 */
    1,				/* isOptional */
    NULL,
  },
};

static SamPnpResource st128RubyPnPResources[]={
  {				/* SamHardwareResPort */
    0,				/* isOptional */
    "ST97PNP",
    4, 0
  },
  {				/* SamHardwareResIrq */
    1,				/* isOptional */
    "ST97PNP",
    4, 0
  },
  {				/* SamHardwareResAuxPort0 */
    1,				/* isOptional */
    NULL,
  },
  {				/* SamHardwareResAuxPort1 */
    1,				/* isOptional */
    NULL,
  },
};

static SamPnpResource maxi64PnPResources[]={
  {				/* SamHardwareResPort */
    0,				/* isOptional */
    "sam9407 (Maxi Sound 64)",
    3, 0
  },
  {				/* SamHardwareResIrq */
    1,				/* isOptional */
    "sam9407 (Maxi Sound 64)",
    3, 0
  },
  {				/* SamHardwareResAuxPort0 */
    1,				/* isOptional */
    NULL,
  },
  {				/* SamHardwareResAuxPort1 */
    1,				/* isOptional */
    NULL,
  },
};

SamPnPInfo samPnPInfos[]={
  {
    0x1211B250,
    SamHardwareTypeEWS64S,
    ews64sPnpResources,
  },
  {
    0x36A8630E,
    SamHardwareTypeEWS64XL,
    ews64xlPnpResources,
  },
  {
    0x3746630E,
    SamHardwareTypeST128,
    st128RubyPnPResources,
  },
  {
    0x68187316,
    SamHardwareTypeMAXI64,
    maxi64PnPResources,
  },
  {
    0				/* sentinel */
  }
};

#ifdef CONFIG_PCI
static SamPCIResource st128PCIResources[]={
  {				/* SamHardwareResPort */
    0,				/* isOptional */
    "ST128DDMA",
    PCI_BASE_ADDRESS_0
  },
  {				/* SamHardwareResIrq */
    1,				/* isOptional */
    "ST128DDMA",
    PCI_INTERRUPT_LINE
  },
  {				/* SamHardwareResAuxPort0 */
    1,				/* isOptional */
    NULL
  },
  {				/* SamHardwareResAuxPort1 */
    1,				/* isOptional */
    NULL
  },
};

SamPCIInfo samPCIInfos[]={
  {
    0x128E,			/* Hoontech Vendor ID */
    0x0009,			/* device id */
    SamHardwareTypeST128DDMA,
    st128PCIResources,
  },
  {
    0				/* sentinel */
  }
};
#endif
