#ifndef _VENDOR_H
#define _VENDOR_H

#include <linux/config.h>

struct SamCard;

typedef enum {
  SamHardwareTypeUnknown,
  SamHardwareTypeEWS64S,
  SamHardwareTypeEWS64XL,
  SamHardwareTypeST128,
  SamHardwareTypeST128DDMA,
  SamHardwareTypeMAXI64,
  SamHardwareTypeSentinel
} SamHardwareType;

typedef enum {
  SamFirmwareTypeGenuine,
  SamFirmwareTypeEWS64,
  SamFirmwareTypeSentinel
} SamFirmwareType;

typedef enum {
  SamMixerTypeUnsupported,
  SamMixerTypeBoolean,
  SamMixerTypeByte,
  SamMixerTypeByteEWS64,	/* used on EWS64, range 0x00-0x80 */
  SamMixerTypeByteEWS64Twice,	/* dto., send twice (e.g. stereo) */
  SamMixerTypeByteEWS647F,	/* used on EWS64, range 0x00-0x7F */
  SamMixerType7Bit,
  SamMixerType3Bit,
  SamMixerType0To2,
  SamMixerTypeWav,
  SamMixerTypeSB,
} SamMixerType;

typedef struct {
  SamMixerType type;
  unsigned char ctrl;
  int storeAt;
  char doesReset;
} SamMixerVendorInfo;

/* buggy firmware doesn't remember all
 * control values needed
 * We remember those values and restore/backup
 * them to SAM9407 when loading/unloading the
 * module
 */
typedef struct {
  unsigned aud_onoff;
} SamEWS64CtrlValues;

typedef struct {
  char *name;
  unsigned probeAddr;
  char *probeStr;
  char verifyChecksum;		/* after firmware load */
  int (*setup)(struct SamCard *card);
  short mmtSlots;
  unsigned char playBackChannels, recordChannels;
  unsigned char getMMTAck;	/* GET_MMT ACK, 0==no ACK */
  unsigned char openAck;	/* W_OPEN ack start, 0==no ACK */
  unsigned char closeAck;	/* W_CLOSE ack start */
  char closeAckRecChannel;	/* W_CLOSE ack sent even for recording channel */
  char volFormatEWS64;		/* volume value range is 0x0000-0x8000 */
  SamMixerVendorInfo *mixerInfo;	/* indexed by SamMixerId */
  unsigned char resetRestoreCtrl;	/* restore this control after firmware reset */
  unsigned nResetReadjust;	/* # of bytes in resetSend */
  unsigned char *resetReadjust;	/* send these controls after a firmware reset */
  unsigned nSBankRehash;	/* # of bytes in sbankRehash */
  unsigned char *sbankRehash;	/* send these controls after soundbank changes */
  unsigned ctrlValuesSize;	/* how many bytes are stored in card->ctrlValues */
} SamFirmwareInfo;

typedef struct {
  char *name;
  int (*reset)(struct SamCard *card);
} SamHardwareInfo;

typedef enum {
  SamHardwareResPort,
  SamHardwareResIrq,
  SamHardwareResAuxPort0,
  SamHardwareResAuxPort1,
  SamHardwareResSentinel
} SamHardwareResType;

typedef struct {
  int value;
  char *id;
} SamHardwareResource;

typedef struct {
  int isOptional;
  char *id;
  int ldn, idx;
} SamPnpResource;

typedef struct {
  unsigned id;			/* board id #, 0==end of list */
  SamHardwareType hardwareType;
  SamPnpResource *resources;	/* indexed by SamHardwareResType */
} SamPnPInfo;

#ifdef CONFIG_PCI
typedef struct {
  int isOptional;
  char *id;
  int where;
} SamPCIResource;

typedef struct {
  unsigned short vendor, devId;
  SamHardwareType hardwareType;
  SamPCIResource *resources;	/* indexed by SamHardwareResType */
} SamPCIInfo;
#endif

extern SamFirmwareInfo samFirmwareInfos[];
extern SamHardwareInfo samHardwareInfos[];

extern SamPnPInfo samPnPInfos[];

#ifdef CONFIG_PCI
extern SamPCIInfo samPCIInfos[];
#endif

#endif /* _VENDOR_H */
