/*
  Copyright (C) 1998, 1999 Gerd Rausch <gerd@alf.gun.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include "x9407.h"

#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

static void mixerNotify(XtPointer closure, int *source, XtInputId *inputId)
{
  KnownDevice *knownDevice;
  char dirtyIds[SAM_MIXER_ID_LAST+2];
  SamMixerValue mixerValues[SAM_MIXER_ID_LAST+1];
  int n, i, nQuery, id;
  MixerCallback *callback;

  knownDevice=applState.knownDevices+applState.deviceUsed;

  if((n=read(applState.mixerFd, dirtyIds, sizeof(dirtyIds)))<0) {
    fprintf(stderr, "fatal: can't read dirty flags from mixer device\n");
    exit(1);
  }

  nQuery=0;
  for(i=0; i<n; i++) {
    if(dirtyIds[i]<0) {
      switch(dirtyIds[i]) {
      case SAM_MIXER_SBANK_CHANGED:
	if(ioctl(applState.mixerFd, SAM_IOC_DRIVER_INFO, &knownDevice->driverInfo)<0) {
	  memset(&applState.sbanks, 0, sizeof(applState.sbanks));
	  error(1, "%s", resources.errQueryDriver);
	}
	if(ioctl(applState.mixerFd, SAM_IOC_BANK_INFO, &applState.sbanks)<0) {
	  memset(&applState.sbanks, 0, sizeof(applState.sbanks));
	  error(1, "%s", resources.errQuerySBank);
	}
	notifySBankChanged();
	break;
      }
    } else
      mixerValues[nQuery++].id=dirtyIds[i];
  }
  mixerValues[nQuery].id=SAM_MIXER_ID_SENTINEL;
  
  if(ioctl(applState.mixerFd, SAM_IOC_MIXER_GET, mixerValues)>=0) {
    for(i=0; i<nQuery; i++) {
      id=mixerValues[i].id;
      applState.mixer[id].value=mixerValues[i].value;
      for(callback=applState.mixer[id].callbacks; callback; callback=callback->next)
	(*callback->callback)(id, callback);
    }
  } else
    error(1, "%s", resources.errQueryMixer);
}

void closeDevice(void)
{
  if(applState.deviceUsed>=0) {
    XtRemoveInput(applState.inputId);
    close(applState.mixerFd);
    applState.deviceUsed=applState.mixerFd=-1;

    if(applState.fsConfigPath) {
      XtFree(applState.fsConfigPath);
      applState.fsConfigPath=NULL;
    }

    setDeviceTitle();
  }
}

void openDevice(int card)
{
  KnownDevice *knownDevice;
  SamMixerValue mixerValues[SAM_MIXER_ID_LAST];
  int i;

  closeDevice();

  if(card<0 || card>=MAX_KNOWN_DEVICES ||
     !applState.knownDevices[card].device)
    return;

  if((applState.mixerFd=open(applState.knownDevices[card].device, O_RDWR))<0) {
    error(1, "%s: %s", resources.errOpenDev, applState.knownDevices[card].device);
    return;
  }

  applState.deviceUsed=card;
  applState.inputId=XtAppAddInput(applState.appContext, applState.mixerFd, (XtPointer)XtInputReadMask, mixerNotify, NULL);

  setDeviceTitle();

  if(!*applState.knownDevices[applState.deviceUsed].driverInfo.firmware)
    return;

  knownDevice=applState.knownDevices+applState.deviceUsed;
  if(ioctl(applState.mixerFd, SAM_IOC_DRIVER_INFO, &knownDevice->driverInfo)<0) {
    memset(&applState.sbanks, 0, sizeof(applState.sbanks));
    error(1, "%s", resources.errQueryDriver);
  }

  if(ioctl(applState.mixerFd, SAM_IOC_BANK_INFO, &applState.sbanks)<0) {
    memset(&applState.sbanks, 0, sizeof(applState.sbanks));
    error(1, "%s", resources.errQuerySBank);
  }

  for(i=1; i<SAM_MIXER_ID_LAST; i++) {
    applState.mixer[i].supported=applState.mixer[i].value=0;
    mixerValues[i-1].id=i;
    mixerValues[i-1].value=0;
  }
  mixerValues[SAM_MIXER_ID_LAST-1].id=SAM_MIXER_ID_SENTINEL;

  if(ioctl(applState.mixerFd, SAM_IOC_MIXER_INFO, mixerValues)<0)
    error(1, "%s", resources.errQueryMixer);

  for(i=1; i<SAM_MIXER_ID_LAST; i++) {
    if(mixerValues[i-1].value)
      applState.mixer[i].supported=1;
  }

  if(ioctl(applState.mixerFd, SAM_IOC_MIXER_GET, mixerValues)<0)
    error(1, "%s", resources.errQueryMixer);

  for(i=1; i<SAM_MIXER_ID_LAST; i++) {
    if(applState.mixer[i].supported)
      applState.mixer[i].value=mixerValues[i-1].value;
    else
      applState.mixer[i].value=0;
  }
}

void scanDevices(int card)
{
  int cardNr, fd;
  char devName[256];
  SamDriverInfo driverInfo;
  KnownDevice *knownDevice;

  closeDevice();

  for(cardNr=0; cardNr<MAX_KNOWN_DEVICES; cardNr++) {
    knownDevice=applState.knownDevices+cardNr;
    if(knownDevice->device)
      XtFree(knownDevice->device);
  }
  memset(applState.knownDevices, 0, sizeof(applState.knownDevices));

  for(cardNr=0; cardNr<MAX_KNOWN_DEVICES; cardNr++) {
    knownDevice=applState.knownDevices+cardNr;

    sprintf(devName, "/dev/sam%d_mixer", cardNr);
    if((fd=open(devName, O_RDWR))<0) {
      if(cardNr!=0)
	break;
      strcpy(devName, "/dev/mixer");
      if((fd=open(devName, O_RDWR))<0)
	break;
    }

    if(ioctl(fd, SAM_IOC_DRIVER_INFO, &driverInfo)<0) {
      close(fd);
      break;
    }
    knownDevice->device=XtNewString(devName);
    memcpy(&knownDevice->driverInfo, &driverInfo, sizeof(SamDriverInfo));
    
    close(fd);
  }

  openDevice(card);
}

void changeMixerValues(SamMixerValue *values)
{
  if(ioctl(applState.mixerFd, SAM_IOC_MIXER_SET, values)<0)
    error(1, "%s", resources.errSetMixer);
}

void changeMixerValue(int id, unsigned value)
{
  int n, i;
  SamMixerValue backupValues[SAM_MIXER_ID_LAST];
  SamMixerValue mixerValues[2];
 
  if(applState.mixer[id].doesReset) {
    /* if the requested control does
     * reset firmware, we backup all
     * mixer values and restore them later on
     */
    n=0;
    for(i=1; i<SAM_MIXER_ID_LAST; i++)
      if(!applState.mixer[i].doesReset &&
	 applState.mixer[i].supported) {
	backupValues[n].id=i;
	backupValues[n].value=applState.mixer[i].value;
	n++;
      }
    backupValues[n].id=SAM_MIXER_ID_SENTINEL;
  }

  mixerValues[0].id=id;
  mixerValues[0].value=value;
  mixerValues[1].id=SAM_MIXER_ID_SENTINEL;

  changeMixerValues(mixerValues);

  if(applState.mixer[id].doesReset)
    changeMixerValues(backupValues);
}

void loadFirmware(char *fileName)
{
  SamFirmware firmware;
  int fd;
  struct stat st;

  if((fd=open(fileName, O_RDONLY))<0) {
    error(1, "%s: %s", resources.errOpenFirm, fileName);
    return;
  }
  if(fstat(fd, &st)<0) {
    error(1, "%s: %s", resources.errStatFirm, fileName);
    close(fd);
    return;
  }
  if((firmware.data=(unsigned short *)mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0))==MAP_FAILED) {
    error(1, "%s: %s", resources.errMMapFirm, fileName);
    close(fd);
    return;
  }
  firmware.size=st.st_size>>1;

  if(ioctl(applState.mixerFd, SAM_IOC_FIRMWARE_LOAD, &firmware)<0) {
    closeDevice();
    error(1, "%s", resources.errLoadFirm);
  }

  scanDevices(applState.deviceUsed);

  munmap((void *)firmware.data, st.st_size);
  close(fd);
}

void loadSoundbank(int id, char *fileName)
{
  SamSoundBank soundBank;
  int fd;
  struct stat st;

  if((fd=open(fileName, O_RDONLY))<0) {
    error(1, "%s: %s", resources.errOpenSBank, fileName);
    return;
  }
  if(fstat(fd, &st)<0) {
    error(1, "%s: %s", resources.errStatSBank, fileName);
    close(fd);
    return;
  }
  if((soundBank.data=(char *)mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0))==MAP_FAILED) {
    error(1, "%s: %s", resources.errMMapSBank, fileName);
    close(fd);
    return;
  }
  soundBank.id=id;
  soundBank.size=st.st_size;

  if(ioctl(applState.mixerFd, SAM_IOC_BANK_LOAD, &soundBank)<0)
    error(1, "%s", resources.errLoadSBank);

  munmap((void *)soundBank.data, st.st_size);
  close(fd);
}

void unloadSoundbank(int id)
{
  if(ioctl(applState.mixerFd, SAM_IOC_BANK_UNLOAD, &id)<0)
    error(1, "%s", resources.errUnloadSBank);
}

void changeSBankPriority(int id, int priority)
{
  SamSoundBankPrio prio;
  prio.id=id;
  prio.priority=priority;

  if(ioctl(applState.mixerFd, SAM_IOC_BANK_PRIO_CHG, &prio)<0)
    error(1, "%s", resources.errSBankPrioChg);
}
