/*
  Copyright (C) 1998, 1999 Gerd Rausch <gerd@alf.gun.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw3d/MenuButton.h>
#include <X11/Xaw3d/SimpleMenu.h>
#include <X11/Xaw3d/SmeBSB.h>

#include <X11/xpm.h>

#include "Tab.h"
#include "Layout.h"

#include <mixerctl.h>

#include "x9407.h"

#include "x9407.xpm"

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

typedef struct Menu {
  char *name;
  XtCallbackProc callback;
  struct Menu *subMenu;
} Menu;

static void quitAction(Widget w, XEvent *event, String *params, Cardinal *numParams);
static void dismissAction(Widget w, XEvent *event, String *params, Cardinal *numParams);
static void ackTextAction(Widget w, XEvent *event, String *params, Cardinal *numParams);
static void copyrightCallback(Widget w, XtPointer closure, XtPointer callData);
static void quitCallback(Widget w, XtPointer closure, XtPointer callData);
static void dismissCallback(Widget w, XtPointer closure, XtPointer callData);
static void loadCallback(Widget w, XtPointer closure, XtPointer callData);
static void saveCallback(Widget w, XtPointer closure, XtPointer callData);

ApplState applState;

Resources resources;

static Menu fileMenu[]={
  {
    "load",
    loadCallback
  },
  {
    "save",
    saveCallback
  },
  {
    "copyright",
    copyrightCallback
  },
  {
    "quit",
    quitCallback
  },
  {
    NULL		/* sentinel */
  }
};

static Menu mainMenu[]={
  {
    "file",
    NULL,
    fileMenu
  },
  {
    NULL		/* sentinel */
  }
};

static XrmOptionDescRec x9407Options[] = {
  {"-card", "card", XrmoptionSepArg, NULL},
};

static XtResource x9407Resources[] = {
  {
    "card", "Card",
    XtRInt, sizeof(int),
    XtOffsetOf(Resources, card),
    XtRInt, (XtPointer)0
  },
  {
    "adjustRelatedButton", "AdjustRelatedButton",
    XtRInt, sizeof(int),
    XtOffsetOf(Resources, adjustRelatedButton),
    XtRInt, (XtPointer)2
  },
  {
    "errorMsgTimeout", "ErrorMsgTimeout",
    XtRInt, sizeof(int),
    XtOffsetOf(Resources, errorMsgTimeout),
    XtRInt, (XtPointer)2000
  },
  {
    "applTitle", "ApplTitle",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, applTitle),
    XtRString, "x9407"
  },
  {
    "fsLoadTitle", "FSLoadTitle",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, fsLoadTitle),
    XtRString, "Load Settings"
  },
  {
    "fsSaveTitle", "FSSaveTitle",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, fsSaveTitle),
    XtRString, "Save Settings"
  },
  {
    "fsFirmTitle", "FSFirmTitle",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, fsFirmTitle),
    XtRString, "Load Firmware"
  },
  {
    "fsSBankTitle", "FSSBankTitle",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, fsSBankTitle),
    XtRString, "Load Soundbank"
  },
  {
    "errNoCardSel", "ErrNoCardSel",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errNoCardSel),
    XtRString, "No card selected"
  },
  {
    "errQueryDriver", "ErrQueryDriver",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errQueryDriver),
    XtRString, "Can't query driver info"
  },
  {
    "errQuerySBank", "ErrQuerySBank",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errQuerySBank),
    XtRString, "Can't query soundbank info"
  },
  {
    "errQueryMixer", "ErrQueryMixer",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errQueryMixer),
    XtRString, "Can't query mixer info"
  },
  {
    "errSetMixer", "ErrSetMixer",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errSetMixer),
    XtRString, "Can't set mixer values"
  },
  {
    "errOpenFile", "ErrOpenFile",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errOpenFile),
    XtRString, "Can't open file"
  },
  {
    "errOpenDev", "ErrOpenDev",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errOpenDev),
    XtRString, "Can't open device"
  },
  {
    "errOpenFirm", "ErrOpenFirm",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errOpenFirm),
    XtRString, "Can't open firmware"
  },
  {
    "errStatFirm", "ErrStatFirm",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errStatFirm),
    XtRString, "Can't stat firmware"
  },
  {
    "errMMapFirm", "ErrMMapFirm",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errMMapFirm),
    XtRString, "Can't mmap firmware"
  },
  {
    "errLoadFirm", "ErrLoadFirm",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errLoadFirm),
    XtRString, "Can't load firmware"
  },
  {
    "errOpenSBank", "ErrOpenSBank",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errOpenSBank),
    XtRString, "Can't open soundbank"
  },
  {
    "errStatSBank", "ErrStatSBank",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errStatSBank),
    XtRString, "Can't stat soundbank"
  },
  {
    "errMMapSBank", "ErrMMapSBank",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errMMapSBank),
    XtRString, "Can't mmap soundbank"
  },
  {
    "errLoadSBank", "ErrLoadSBank",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errLoadSBank),
    XtRString, "Can't load soundbank"
  },
  {
    "errUnloadSBank", "ErrUnloadSBank",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errUnloadSBank),
    XtRString, "Can't unload soundbank"
  },
  {
    "errSBankPrioChg", "ErrSBankPrioChg",
    XtRString, sizeof(String),
    XtOffsetOf(Resources, errSBankPrioChg),
    XtRString, "Can't change soundbank priority"
  },
};

static XtActionsRec x9407Actions[] = {
  { "quit", quitAction },
  { "dismiss", dismissAction },
  { "ackText", ackTextAction },
};

static void buildMenu(Widget top, Menu *aMenu)
{
  Menu *menu;
  Widget w;
  char buf[256];
  for(menu=aMenu; menu->name; menu++) {
    if(menu->subMenu) {
      sprintf(buf, "%s_menu", menu->name);
      w=XtVaCreatePopupShell(buf, simpleMenuWidgetClass, top,
			     NULL);
      buildMenu(w, menu->subMenu);
      w=XtVaCreateManagedWidget(menu->name, menuButtonWidgetClass, top,
				XtNmenuName, XtNewString(buf),
				NULL);
    } else
      w=XtVaCreateManagedWidget(menu->name, smeBSBObjectClass, top,
				NULL);
    if(menu->callback)
      XtAddCallback(w, XtNcallback, menu->callback, NULL);
  }
}

static void createMixerAssocs(void)
{
  Display *dpy=XtDisplay(applState.top);
  SamMixerAssoc *assoc;
  XrmQuark quark;

  for(assoc=samMixerAssoc; assoc->name; assoc++) {
    applState.mixer[assoc->id].doesReset=assoc->doesReset;
    quark=XrmStringToQuark(assoc->name);
    XSaveContext(dpy, (XID)quark, applState.mixerAssocContext,
		 (XPointer)assoc);
  }
}

int main(int argc, char **argv)
{
  Widget main, tab;
  Pixmap iconPixmap, iconMask;

  applState.deviceUsed=-1;
  applState.top=XtVaAppInitialize(&applState.appContext, "X9407", x9407Options, XtNumber(x9407Options), &argc, argv, fallbackResources,
				  NULL);

  XtVaGetValues(applState.top, XtNiconPixmap, &iconPixmap, NULL);
  if(!iconPixmap) {
    if(XpmCreatePixmapFromData(XtDisplay(applState.top),
			       RootWindowOfScreen(XtScreen(applState.top)),
			       x9407_xpm, &iconPixmap, &iconMask, NULL)==XpmSuccess)
      XtVaSetValues(applState.top,
		    XtNiconPixmap, iconPixmap,
		    XtNiconMask, iconMask,
		    NULL);
  }

  XtVaGetApplicationResources(applState.top, &resources,
			      x9407Resources, XtNumber(x9407Resources),
			      NULL);

  applState.mixerAssocContext=XUniqueContext();
  applState.ackTextContext=XUniqueContext();
  applState.ackTextClosureContext=XUniqueContext();

  createMixerAssocs();

  scanDevices(resources.card);

  XtAppAddActions(applState.appContext, x9407Actions, XtNumber(x9407Actions));
 
  main=XtVaCreateManagedWidget("main", layoutWidgetClass, applState.top,
			       NULL);

  /* create menu bar */
  buildMenu(main, mainMenu);

  tab=XtVaCreateManagedWidget("tab", tabWidgetClass, main,
			      NULL);
  addPanes(tab);

  applState.msg=XtVaCreateManagedWidget("msg", labelWidgetClass, main,
					XtNlabel, "",
					NULL);

  XtOverrideTranslations(applState.top, XtParseTranslationTable ("<Message>WM_PROTOCOLS: quit()"));

  XtRealizeWidget(applState.top);

  applState.wmDeleteWindow=XInternAtom(XtDisplay(applState.top), "WM_DELETE_WINDOW", False);
  XSetWMProtocols(XtDisplay(applState.top), XtWindow(applState.top), &applState.wmDeleteWindow, 1);

  XtAppMainLoop(applState.appContext);

  return 0;
}

static void quitAction(Widget w, XEvent *event, String *params, Cardinal *numParams)
{
  exit(0);
}

static void dismissAction(Widget w, XEvent *event, String *params, Cardinal *numParams)
{
  XtUnmapWidget(w);
}

static void copyrightCallback(Widget w, XtPointer closure, XtPointer callData)
{
  static Widget copyright;
  Widget layout, dismiss;
  if(!copyright) {
    copyright=XtVaAppCreateShell("copyright", "Copyright",
				 transientShellWidgetClass, XtDisplay(applState.top),
				 XtNtransientFor, applState.top,
				 NULL);
    layout=XtVaCreateManagedWidget("layout", layoutWidgetClass, copyright,
				   NULL);
    XtVaCreateManagedWidget("msg", labelWidgetClass, layout,
			    NULL);
    dismiss=XtVaCreateManagedWidget("dismiss", commandWidgetClass, layout,
				    NULL);
    XtAddCallback(dismiss, XtNcallback, dismissCallback, NULL);
    XtOverrideTranslations(copyright, XtParseTranslationTable ("<Message>WM_PROTOCOLS: dismiss()"));
    XtRealizeWidget(copyright);

    XSetWMProtocols(XtDisplay(copyright), XtWindow(copyright), &applState.wmDeleteWindow, 1);
  }
  XtMapWidget(copyright);
}

static void quitCallback(Widget w, XtPointer closure, XtPointer callData)
{
  exit(0);
}

static void dismissCallback(Widget w, XtPointer closure, XtPointer callData)
{
  Widget shell;
  for(shell=w; !XtIsShell(shell); shell=XtParent(shell));
  XtUnmapWidget(shell);
}

static char *getLoadSaveDefaultFile(void)
{
  char *initPath;
  static char buf[MAXPATHLEN];
  if((initPath=getenv("HOME"))) {
    sprintf(buf, "%s/.sam9407_%d", initPath, applState.deviceUsed);
    initPath=buf;
  }
  return initPath;
}

static void loadCallback(Widget w, XtPointer closure, XtPointer callData)
{
  char *fileName;
  FILE *fp;
  SamMixerAssoc *assoc;
  XrmQuark quark;
  char buf[256], *cp, *name, *value;
  SamMixerValue resetValues[SAM_MIXER_ID_LAST], noResetValues[SAM_MIXER_ID_LAST], *mixerValue;
  int pass, i, j;
  unsigned v;

  if(applState.deviceUsed<0) {
    error(0, "%s", resources.errNoCardSel);
    return;
  }

  if((fileName=XsraSelFile(applState.top, resources.fsLoadTitle, getLoadSaveDefaultFile(), &applState.fsConfigPath, 0, NULL))) {
    if(!(fp=fopen(fileName, "r"))) {
      error(0, "%s: %s", resources.errOpenFile, fileName);
      XtFree(fileName);
      return;
    }
    memset(&resetValues, 0, sizeof(resetValues));
    memset(&noResetValues, 0, sizeof(noResetValues));
    while(fgets(buf, sizeof(buf), fp)) {
      cp=buf;
      while(isspace(*cp)) cp++;
      if(!*cp || *cp=='#')
	continue;
      name=cp;
      while(*cp && !isspace(*cp)) cp++;
      if(*cp)
	*cp++=0;
      quark=XrmStringToQuark(name);
      if(XFindContext(XtDisplay(applState.top), (XID)quark,
		      applState.mixerAssocContext, (XPointer *)&assoc)!=0)
	continue;
      /* skip spaces at the beginning of value */
      while(isspace(*cp)) cp++;
      value=cp;
      /* find the end of value */
      while(*cp && !isspace(*cp)) cp++;
      *cp=0;

      v=strtol(value, NULL, 0);
      if(v==applState.mixer[assoc->id].value)
	continue;

      mixerValue=assoc->doesReset ? resetValues+assoc->id : noResetValues+assoc->id;

      mixerValue->id=assoc->id;
      mixerValue->value=v;
    }
    /* throw out unused slots in resetValues and noResetValues */
    for(pass=0; pass<2; pass++) {
      mixerValue=pass ? noResetValues : resetValues;
      j=0;
      for(i=0; i<SAM_MIXER_ID_LAST; i++) {
	if(mixerValue[i].id!=SAM_MIXER_ID_SENTINEL) {
	  if(i!=j)
	    memmove(mixerValue+j, mixerValue+i, sizeof(SamMixerValue));
	  j++;
	}
	mixerValue[j].id=SAM_MIXER_ID_SENTINEL;
      }
    }
    
    fclose(fp);
    XtFree(fileName);

    /* set reset values first, then noResetValues */
    changeMixerValues(resetValues);
    changeMixerValues(noResetValues);
  }
}

static void saveCallback(Widget w, XtPointer closure, XtPointer callData)
{
  char *fileName;
  FILE *fp;
  SamMixerAssoc *assoc;
  XrmQuark quark;

  if(applState.deviceUsed<0) {
    error(0, "%s", resources.errNoCardSel);
    return;
  }

  if((fileName=XsraSelFile(applState.top, resources.fsSaveTitle, getLoadSaveDefaultFile(), &applState.fsConfigPath, 0, NULL))) {
    if(!(fp=fopen(fileName, "w"))) {
      error(0, "%s: %s", resources.errOpenFile, fileName);
      XtFree(fileName);
      return;
    }
    for(assoc=samMixerAssoc; assoc->name; assoc++)
      fprintf(fp, "%-16s %d\n", assoc->name, applState.mixer[assoc->id].value);
    fclose(fp);
    XtFree(fileName);
  }
}

static void eraseMessage(XtPointer closure, XtIntervalId *id)
{
  XtVaSetValues(applState.msg, XtNlabel, "", NULL);
}

void error(int perr, char *format, ...)
{
  char buf[1024];
  int errn=errno, n;
  va_list ap;

  if(!applState.msg)
    return;

  if(applState.msgTimeoutId) {
    XtRemoveTimeOut(applState.msgTimeoutId);
    applState.msgTimeoutId=0;
  }

  va_start(ap, format);
  n=vsprintf(buf, format, ap);
  va_end(ap);

  if(perr)
    sprintf(buf+n, ": %s", strerror(errn));

  XtVaSetValues(applState.msg, XtNlabel, buf, NULL);

  applState.msgTimeoutId=XtAppAddTimeOut(applState.appContext, resources.errorMsgTimeout, eraseMessage, NULL);
}

void setDeviceTitle(void)
{
  static char *oldTitle;
  char *newTitle;
  if(applState.deviceUsed>=0) {
    newTitle=XtMalloc(strlen(resources.applTitle)+
		      strlen(applState.knownDevices[applState.deviceUsed].driverInfo.hardware)+4);
    sprintf(newTitle, "%s (%s)", resources.applTitle,
	    applState.knownDevices[applState.deviceUsed].driverInfo.hardware);
  } else {
    newTitle=XtMalloc(strlen(resources.applTitle)+1);
    strcpy(newTitle, resources.applTitle);
  }
  XtVaSetValues(applState.top,
		XtNtitle, newTitle,
		XtNiconName, newTitle,
		NULL);
  if(oldTitle)
    XtFree(oldTitle);
  oldTitle=newTitle;
}

static void ackTextAction(Widget w, XEvent *event, String *params, Cardinal *numParams)
{
  AckTextCallback callback=NULL;
  XtPointer closure=NULL;

  if(XFindContext(XtDisplay(w), (XID)w, applState.ackTextContext, (XPointer *)&callback)==0 &&
     callback) {
    XFindContext(XtDisplay(w), (XID)w, applState.ackTextClosureContext, (XPointer *)&closure);
    (*callback)(w, closure);
  }
}
