#ifndef _X9407_H
#define _X9407_H

#include <sam9407.h>
#include <X11/Intrinsic.h>

#define MAX_KNOWN_DEVICES	8

typedef void (*AckTextCallback)(Widget w, XtPointer closure);

typedef struct {
  char *device;
  SamDriverInfo driverInfo;
} KnownDevice;

typedef struct MixerCallback {
  void (*callback)(int id, struct MixerCallback *info);
  struct MixerCallback *next;
  XtPointer closure;
} MixerCallback;
typedef struct {
  int doesReset, supported;
  unsigned value;
  MixerCallback *callbacks;
} MixerInfo;

typedef struct {
  int deviceUsed, mixerFd;
  XtAppContext appContext;
  XContext mixerAssocContext, ackTextContext, ackTextClosureContext;
  Atom wmDeleteWindow;
  XtInputId inputId;
  XtIntervalId msgTimeoutId;
  Widget top, msg;
  KnownDevice knownDevices[MAX_KNOWN_DEVICES];
  SamSoundBankInfo sbanks[SAM_NROF_SBANKS];
  MixerInfo mixer[SAM_MIXER_ID_LAST];
  String fsFirmwarePath, fsSoundbankPath, fsConfigPath;
} ApplState;

typedef struct {
  int card, adjustRelatedButton, errorMsgTimeout;
  String applTitle, fsLoadTitle, fsSaveTitle, fsFirmTitle, fsSBankTitle;
  /* several error messages */
  String errNoCardSel, errQueryDriver, errQuerySBank, errQueryMixer, errSetMixer;
  String errOpenFile, errOpenDev;
  String errOpenFirm, errStatFirm, errMMapFirm, errLoadFirm;
  String errOpenSBank, errStatSBank, errMMapSBank, errLoadSBank;
  String errUnloadSBank, errSBankPrioChg;
} Resources;

extern ApplState applState;
extern Resources resources;
extern char *fallbackResources[];

void error(int perr, char *format, ...);
void setDeviceTitle(void);

void scanDevices(int card);
void closeDevice(void);
void openDevice(int card);
void changeMixerValues(SamMixerValue *values);
void changeMixerValue(int id, unsigned value);
void loadFirmware(char *fileName);
void loadSoundbank(int id, char *fileName);
void unloadSoundbank(int id);
void changeSBankPriority(int id, int priority);

void addPanes(Widget parent);
void notifySBankChanged(void);

char *XsraSelFile();

#endif /* _X9407_H */
