#include <stdio.h>

#include <asm/io.h>

typedef int BOOL;
typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long  DWORD;
typedef char *LPSTR;


#include "controls.h"
#include "sam9407.h"

//------------------------------------------------------------------------

#define IN_BYTE(p)	inb(p)
#define OUT_BYTE(p,v)	outb(v,p)
#define IN_WORD(p)	inw(p)
#define OUT_WORD(p,v)	outw(v,p)
#define STI
#define CLI
//------------------------------------------------------------------------

WORD MPUBASE = 0x330;
static int gData, gCmd;

WORD SBPort = 0x220;


//========================================================================
//
//	Basic IO
//
//========================================================================
static int Hw_Error( int line, char *p )
{
	printf("\n[%d] %s\n",line,p );
	return -1;
}

int S94_GetStatus( void )
{
	 return IN_BYTE(CNTL_PORT_MPU);
}

static int S94_IsReady_ACK( int id, int ack )
{
	unsigned c=0x100;
	unsigned i=0x100;
	int r=0,s=0;

	do {
		if( --c == 0 )
		{
			if( --i == 0 )
			{
			Hw_Error( s, "MPU401 ACK id Failed" );
			return 0;
			}
			else c=0x100;
		}

		s = S94_GetStatus();
	}
	while( (s&id)!=id ) ;


	c=0x3;

	do {
		if( --c == 0 )
		{
			Hw_Error( __LINE__, "MPU401 ACK data Failed" );
			return 0;
		}
		r = S94_DataRet();
		if( r==0xAB )
		{
			Hw_Error( __LINE__, "I don't know why ABh ??" );
			while(r==0xAB)
			{
				r = S94_DataRet();
			}
		}
	}
	while( r!=ack ) ;

	return 1;
}

static int S94_IsReady_RF( void )
{
	int c=0x100;

	do {
		if( --c == 0 )
		{
			Hw_Error( __LINE__, "MPU401 RF Error ""No Responding.. Too long" );
			return 0;
		}
	}
	while( ((gCmd=IN_BYTE(CNTL_PORT_MPU))&0x40) );
	return 1;
}

static int S94_IsReady_TE( void )
{
	int c=0x100;
	int i=0x100;

	do {
		if( --c == 0 )
		{
			if( --i == 0 )
			{
			Hw_Error( gCmd, "MPU401 TE Error ""MSB still 1, it must be 0 for read" );
			return 0;
			}
			else c=0x100;
		}
		;
	}
	while( ((gCmd=IN_BYTE(CNTL_PORT_MPU))&0x80) );
	return 1;
}

static int S94_WaitforLow( void )
{
	int c=0x100;

	while( !(gCmd=IN_BYTE(CNTL_PORT_MPU)&0x80) )
	{
		// clear FIFO
		gData=IN_BYTE(DATA_PORT_MPU);
		if( --c == 0 )
		{
			Hw_Error( __LINE__, "MPU401 LOW Error""MSB still 0, it must be 1 for write!" );
			return 0;
		}
	} ;
	return 1;
}

static void S94_CtrlCmd( int cntl )
{
	if( S94_WaitforLow() )
		OUT_BYTE ( CNTL_PORT_MPU, (cntl&0xFF) );
}

static void S94_CtrlData( unsigned int data )
{
	if( S94_IsReady_RF() )
		OUT_BYTE ( DATA_PORT_MPU, (data&0xFF) );
}

int S94_DataRet( void )
{
	if( S94_IsReady_TE() )
			return gData=IN_BYTE ( DATA_PORT_MPU );

	else return 0xff;
}

void S94_SetGenDev( int cntl, unsigned int data )
{
	OUT_BYTE ( CNTL_PORT_MPU, (cntl&0xFF) );
	OUT_BYTE ( DATA_PORT_MPU, (data&0xFF) );
}

void S94_SetGenDevACK( int cntl, unsigned int data )
{
	S94_UART();

	OUT_BYTE ( CNTL_PORT_MPU, (cntl&0xFF) );
	OUT_BYTE ( DATA_PORT_MPU, (data&0xFF) );

	S94_IsReady_ACK( IRQ_BY_GEN, 0x00 );

	S94_WaitforLow();

	S94_UART();
}


void S94_SetWordGen( int nReg, int Channel, int Data )
{
	S94_CtrlCmd( nReg );
	S94_CtrlData( Channel );
	S94_CtrlData( Data );
	S94_CtrlData( Data>>8 );
}

void S94_SetOnOff( int nReg, int Data )
{
	S94_SetGenDev( nReg, (Data)?0x7F : 0 );
}

void S94_SetOnOffAc( int nReg, int Data )
{
	S94_SetGenDevACK( nReg, (Data)?0x7F : 0 );
}


void S94_SetEquType(int reg, int data )
{
	S94_SetGenDev( reg, (data)?0 : 2 );
}


int S94_UART( void )
{
	int c=0x100;
	long i,j=0;

	CLI

	do {
		OUT_BYTE ( CNTL_PORT_MPU, 0x3F );
		//S94_CtrlCmd( 0x3f );
		for(i=0; i<0x100; i++) j++;
		if( --c == 0 )
		{
			Hw_Error( __LINE__, "MPU401 UART""No Responding.. Too long Error");
			return 0;
		}
	} while( (S94_DataRet())!=0xFE ) ;

	S94_WaitforLow();

	STI

	return 1;
}

void S94_StandAlone( void )
{
	OUT_BYTE ( CNTL_PORT_MPU, 0x3F );
	S94_WaitforLow();
}

//========================================================================
//
//	W A V Functions
//
//========================================================================

void S94_Gen_Int( void )
{
	S94_CtrlCmd( GEN_INT );
	S94_CtrlData( 0 );
}

void Wav_Open ( int Channel, int IsMONOorSTR, int Is8or16, int Rate )
{
	int fmt;

	Is8or16 	&= 1;
	IsMONOorSTR &= 1;
	fmt = (IsMONOorSTR<<1) | Is8or16;

	S94_CtrlCmd( W_OPEN );
	S94_CtrlData( Channel );
	S94_CtrlData( fmt );
	S94_CtrlData( Rate );
	S94_CtrlData( Rate>>8 );
}

void Wav_OutClose ( int Channel )
{
	S94_CtrlCmd( W_CLOSE );
	S94_CtrlData( Channel );
}

void Wav_InClose ( int Channel )
{
	S94_CtrlCmd( W_CLOSE );
	S94_CtrlData( Channel );
}

void Wav_Start ( int Channel )
{
	S94_CtrlCmd( W_START );
	S94_CtrlData( Channel );
}

void Wav_End_Xfer ( int Channel )
{
	S94_CtrlCmd( END_XFER );
	S94_CtrlData( Channel );
}

void Wav_Pitch ( int Channel, unsigned int Rate )
{
	S94_CtrlCmd( W_PITCH );
	S94_CtrlData( Channel );
	S94_CtrlData( Rate );
	S94_CtrlData( Rate>>8 );
}

void Wav_VolLeft ( int Channel, unsigned int Volume )
{
	S94_CtrlCmd( W_VOLLEFT );

	S94_CtrlData( Channel );
	S94_CtrlData( Volume );
	S94_CtrlData( Volume>>8 );
}

void Wav_VolRight ( int Channel, unsigned int Volume )
{
	S94_CtrlCmd( W_VOLRIGHT );

	S94_CtrlData( Channel );
	S94_CtrlData( Volume );
	S94_CtrlData( Volume>>8 );
}

void Wav_VolAuxLeft ( int Channel, unsigned int Volume )
{
	S94_CtrlCmd( W_VOLAUXLEFT );

	S94_CtrlData( Channel );
	S94_CtrlData( Volume );
	S94_CtrlData( Volume>>8 );
}

void Wav_VolAuxRight ( int Channel, unsigned int Volume )
{
	S94_CtrlCmd( W_VOLAUXRIGHT );

	S94_CtrlData( Channel );
	S94_CtrlData( Volume );
	S94_CtrlData( Volume>>8 );
}

void Wav_Filt_FC ( int Channel, unsigned int Volume )
{
	S94_CtrlCmd( W_FILT_FC );

	S94_CtrlData( Channel );
	S94_CtrlData( Volume );
	S94_CtrlData( Volume>>8 );
}

void Wav_Filt_Q ( int Channel, unsigned int Volume )
{
	S94_CtrlCmd( W_FILT_Q );

	S94_CtrlData( Channel );
	S94_CtrlData( Volume );
	S94_CtrlData( Volume>>8 );
}

//========================================================================
//
//	M O D Functions
//
//========================================================================

int Mod_Get_Voi( void )
{
	int r;

	CLI

	S94_CtrlCmd( GET_VOI );
	S94_CtrlData( 0 );
	r = S94_DataRet();

	S94_WaitforLow();

	STI

	return r;
}

void Mod_Voi_Open( int Voice )
{
	S94_CtrlCmd( VOI_OPEN );
	S94_CtrlData( Voice );
}

void Mod_Voi_Close( int Voice )
{
	S94_CtrlCmd( VOI_CLOSE );
	S94_CtrlData( Voice );
}

void Mod_Voi_Start( int Voice )
{
	S94_CtrlCmd( VOI_START );
	S94_CtrlData( Voice );
}

void Mod_Voi_Stop( int Voice )
{
	S94_CtrlCmd( VOI_STOP );
	S94_CtrlData( Voice );
}

void Mod_Voi_Vol( int Voice, int Vol )
{
	S94_CtrlCmd( VOI_VOL );
	S94_CtrlData( Voice );
	S94_CtrlData( Vol );
}

void Mod_Voi_Main ( int Voice, int MainLeft, int MainRight ) //PAN
{
	S94_CtrlCmd( VOI_MAIN );
	S94_CtrlData( Voice );
	S94_CtrlData( MainRight );
	S94_CtrlData( MainLeft );
}

void Mod_Voi_Pitch ( int Voice, int pitch )
{
	S94_CtrlCmd( VOI_PITCH );
	S94_CtrlData( Voice );
	S94_CtrlData( pitch );
	S94_CtrlData( pitch>>8 );
}

void Mod_Voi_Aux ( int Voice, int AuxLeft, int AuxRight ) //FX
{
	S94_CtrlCmd( VOI_AUX );
	S94_CtrlData( Voice );
	S94_CtrlData( AuxRight );
	S94_CtrlData( AuxLeft );
}

void Mod_Voi_Filter ( int Voice, int Q, int Fc )
{
	S94_CtrlCmd( VOI_FILT );
	S94_CtrlData( Voice );
	S94_CtrlData( Q );
	S94_CtrlData( Fc );
}

void Mod_Voi_Mem ( int Voice, int bit8or16, DWORD sstart, DWORD lpstart, DWORD lpend)
{
	S94_CtrlCmd( VOI_MEM );

	S94_CtrlData( Voice );
	S94_CtrlData( bit8or16==16 ? 0 : 0x80 );

	S94_CtrlData( (WORD)(sstart>>18) ) ;
	S94_CtrlData( (WORD)(sstart) );
	S94_CtrlData( (WORD)(sstart>>8) );
	S94_CtrlData( (WORD)((sstart>>16)&3) );

	S94_CtrlData( (WORD)(lpstart>>18) );
	S94_CtrlData( (WORD)(lpstart) );
	S94_CtrlData( (WORD)(lpstart>>8) );
	S94_CtrlData( (WORD)((lpstart>>16)&3) );

	S94_CtrlData( (WORD)(lpend>>18) );
	S94_CtrlData( (WORD)(lpend) ) ;
	S94_CtrlData( (WORD)(lpend>>8) );
	S94_CtrlData( (WORD)((lpend>>16)&3) );
}


void SB_Mixer_Out(unsigned reg, unsigned val)
{
	int v = ((val<<4)|(val&0xf));
	OUT_BYTE(SBPort+0x4, (char)reg);
	OUT_BYTE(SBPort+0x5, (char)v);
}

BYTE SB_Mixer_In(int reg)
{
	OUT_BYTE(SBPort+0x4, (char)reg);
	return (BYTE)IN_BYTE(SBPort+0x5);
}

void Set_SB_Mixer ( int p, int v )
{
	int v2;

	v2 = v&0xF;
	v2 = (v2<<4) | v2;

	switch( p )
	{
	case SB_MASTER 	: SB_Mixer_Out(0x22,v2);break;//master
	case SB_WAV 	: SB_Mixer_Out(0x4 ,v2);break;//wave vol
	case SB_FM 	: SB_Mixer_Out(0x26,v2);break;//fm volume
	case SB_CD 	: SB_Mixer_Out(0x28,v2);break;//cd volume
	case SB_LINE  	: SB_Mixer_Out(0x2e,v2);break;//line volume
	case SB_MIC   	: SB_Mixer_Out(0xa, v );break;//mic-phone volume
	}
}



int SMAX( int reg )
{
	switch (reg)
	{
	case EQ_LBL        :     case EQ_MLBL       :     case EQ_MHBL       :
	case EQ_HBL        :     case EQ_LBR        :     case EQ_MLBR       :
	case EQ_MHBR       :     case EQ_HBR        :     case EQF_LB        :
	case EQF_MLB       :     case EQF_MHB       :     case EQF_HB        :
	case AUD_GAINL     :     case AUD_GAINR     :	  case SUR_DEL       :
	case ECH_LEV       :     case ECH_TIM       :     case ECH_FEED      :
	case GM_PAN        :     case AUDL_PAN      :     case AUDR_PAN      :
	case AUDREV_SEND   :     case CHR_DEL	    :	  case CHR_FEED	     :
	case CHR_RATE      :	 case CHR_DEPTH     :	  case REV_TIME      :
	case REV_FEED      :	 case REV_TYPE      :     case SUR_24        :
	case AUDCHR_SEND   :
		return  0x7f;

	case SUR_VOL       :     case GM_VOL        :     case REV_VOL       :
	case CHR_VOL       :     case AUDL_VOL      :     case AUDR_VOL      :
	case VOI_VOL       :     case VOI_MAIN      :     case VOI_AUX       :
	case VOI_FILT	   :	 case GMREV_SEND    :	  case GMCHR_SEND    :
		return  0xFF;

	case W_VOLLEFT     :     case W_VOLRIGHT    :     case W_VOLAUXLEFT  :
	case W_VOLAUXRIGHT :	 case W_FILT_FC     :     case W_FILT_Q      :
	case W_PITCH	   :
		return 0xffff;

	case VOI_PITCH	   :
		return 0x800;

	case SB_MASTER     :	 case SB_WAV		:	  case SB_FM         :
	case SB_CD         :	 case SB_LINE       :
		return 0xF;

	case SB_MIC        :
		return 0x7;

	case SB_MIDI       :
		return 0x3F;

	case SB_ECHO       :
		return 0x1F;
		

	default			   : return 0xff;
	}
}


int GetRegNoByConID( int id )
{
	switch( id )
	{
	  case MX_TRANS_VAL         : return TRANS_VAL;
	  case MX_TRANS_REVSEND     : return TRANS_REVSEND;
	  case MX_TRANS_CHRSEND     : return TRANS_CHRSEND;
	  case MX_EQ_LBL            : return EQ_LBL;
	  case MX_EQ_MLBL           : return EQ_MLBL;
	  case MX_EQ_MHBL           : return EQ_MHBL;
	  case MX_EQ_HBL            : return EQ_HBL;
	  case MX_EQF_LB            : return EQF_LB;
	  case MX_EQF_MLB           : return EQF_MLB;
	  case MX_EQF_MHB           : return EQF_MHB;
	  case MX_EQF_HB            : return EQF_HB;
	  case MX_AUD_GAINL         : return AUD_GAINL;
//	  case MX_AUD_GAINR         : return AUD_GAINR;
	  case MX_GMREV_SEND        : return GMREV_SEND;
	  case MX_GMCHR_SEND        : return GMCHR_SEND;
	  case MX_AUDREV_SEND       : return AUDREV_SEND;
	  case MX_ECH_LEV           : return ECH_LEV;
	  case MX_ECH_TIM           : return ECH_TIM;
	  case MX_ECH_FEED          : return ECH_FEED;
	  case MX_SUR_VOL           : return SUR_VOL;
	  case MX_SUR_DEL           : return SUR_DEL;
	  case MX_SUR_INP           : return SUR_INP;
	  case MX_AUDL_VOL          : return AUDL_VOL;
	  case MX_AUDR_VOL          : return AUDR_VOL;
//	  case MX_AUDL_PAN          : return AUDL_PAN;
//	  case MX_AUDR_PAN          : return AUDR_PAN;
	  case MX_GM_VOL            : return GM_VOL;
	  case MX_GM_PAN            : return GM_PAN;
	  case MX_REV_VOL           : return REV_VOL;
	  case MX_CHR_VOL           : return CHR_VOL;
	  case MX_CHR_DEL           : return CHR_DEL;
	  case MX_CHR_FEED          : return CHR_FEED;
	  case MX_CHR_RATE          : return CHR_RATE;
	  case MX_CHR_DEPTH         : return CHR_DEPTH;
	  case MX_REV_TIME          : return REV_TIME;
	  case MX_REV_FEED          : return REV_FEED;
	  case MX_WAVE_ASS          : return WAVE_ASS;
	  case MX_MOD_ASS           : return MOD_ASS;
	  case MX_GM_POST           : return GM_POST;
	  case MX_WAVE_POST         : return WAVE_POST;
	  case MX_MOD_POST          : return MOD_POST;
	  case MX_AUDECH_POST       : return AUDECH_POST;
	  case MX_EFF_POST          : return EFF_POST;
	  case MX_ECH_ONOFF         : return ECH_ONOFF;
	  case MX_REV_ONOFF         : return REV_ONOFF;
	  case MX_CHR_ONOFF         : return CHR_ONOFF;
	  case MX_SUR_ONOFF         : return SUR_ONOFF;
	  case MX_AUD_ONOFF         : return AUD_ONOFF;
	  case MX_REC_MODE          : return REC_MODE;
	  case MX_AUD_SEL           : return AUD_SEL;
	  case MX_SUR_24            : return SUR_24;
	  case MX_REV_TYPE          : return REV_TYPE;
	  case MX_CHR_TYPE          : return CHR_TYPE;
	  case MX_EQU_TYPE          : return EQU_TYPE;
	  case MX_TRANS_ONOFF       : return TRANS_ONOFF;
	  case MX_TRANS_GMCH        : return TRANS_GMCH;
	  case MX_W_PITCH           : return W_PITCH;
	  case MX_W_VOLLEFT0        : return W_VOLLEFT;
	  case MX_W_VOLLEFT1        : return W_VOLLEFT;
	  case MX_W_VOLLEFT2        : return W_VOLLEFT;
	  case MX_W_VOLLEFT3        : return W_VOLLEFT;
	  case MX_W_VOLLEFT4        : return W_VOLLEFT;
	  case MX_W_VOLLEFT5        : return W_VOLLEFT;
	  case MX_W_VOLLEFT6        : return W_VOLLEFT;
	  case MX_W_VOLLEFT7        : return W_VOLLEFT;
	  case MX_W_VOLLEFT8        : return W_VOLLEFT;
	  case MX_W_VOLAUXLEFT0     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT1     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT2     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT3     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT4     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT5     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT6     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT7     : return W_VOLAUXLEFT;
	  case MX_W_VOLAUXLEFT8     : return W_VOLAUXLEFT;
	  case MX_W_FILT_FC         : return W_FILT_FC;
	  case MX_W_FILT_Q          : return W_FILT_Q;
	  case MX_VOI_VOL           : return VOI_VOL;
	  case MX_VOI_MAIN          : return VOI_MAIN;
	  case MX_VOI_PITCH         : return VOI_PITCH;
	  case MX_VOI_AUX           : return VOI_AUX;
	  case MX_VOI_FILT          : return VOI_FILT;

	  case MX_ECH_SILENT		: return ECH_ONOFF;
	  case MX_EFF_SILENT        : return REV_ONOFF;
	  case MX_EQU_SILENT        : return EQU_TYPE;
	  case MX_AUDCHR_SEND       : return AUDCHR_SEND;

	  case MX_MASTER_VOL        : return MASTER_VOL;

	  case MX_W_PITCH0          :
	  case MX_W_PITCH1          :
	  case MX_W_PITCH2          :
	  case MX_W_PITCH3          :
	  case MX_W_PITCH4          :
	  case MX_W_PITCH5          :
	  case MX_W_PITCH6          :
	  case MX_W_PITCH7          : return W_PITCH;

	  case MX_W_FC0             :
	  case MX_W_FC1             :
	  case MX_W_FC2             :
	  case MX_W_FC3             :
	  case MX_W_FC4             :
	  case MX_W_FC5             :
	  case MX_W_FC6             :
	  case MX_W_FC7             : return W_FILT_FC;

	  case MX_W_Q0              :
	  case MX_W_Q1              :
	  case MX_W_Q2              :
	  case MX_W_Q3              :
	  case MX_W_Q4              :
	  case MX_W_Q5              :
	  case MX_W_Q6              :
	  case MX_W_Q7              : return W_FILT_Q;

	  default :
		return id;
	}
}

static WORD GetMax( int id )
{
	WORD r=GetRegNoByConID(id);
	if( r==(WORD)-1 ) return r;
	else
	{
		r=SMAX(r);
		if( !r )	return (WORD)-1;
	}
	return r;
}

void MixerMessage( unsigned int id, unsigned int val )
{
	unsigned int v, i;
	unsigned int c;

	c = GetRegNoByConID( id );
	v = val * GetMax( id ) / 100L;

	printf( "%x - Val : %d, %x\n", id, v, v );
	
	switch( id )
	{
		case MX_ECH_SILENT   : S94_SetGenDev( c, v ); break;
		case MX_EFF_SILENT   : S94_SetGenDev( c, v ); break;
		case MX_EQU_SILENT   : S94_SetGenDev( c, v ); break;

		case MX_WAVE_ASS     : 
		case MX_MOD_ASS      : S94_SetOnOffAc( c, v ); break;

		case MX_GM_POST      : 
		case MX_WAVE_POST    : 
		case MX_MOD_POST     : 
		case MX_AUDECH_POST  : 
		case MX_AUD_SEL      : 
		case MX_SUR_24       : 
		case MX_SUR_INP      : S94_SetOnOff( c, v ); break;

		case MX_EFF_POST     : S94_SetGenDev( c, v ); break;

		case MX_ECH_ONOFF    : 
		case MX_REV_ONOFF    : 
		case MX_CHR_ONOFF    : 
		case MX_SUR_ONOFF    : 
		case MX_AUD_ONOFF    : S94_SetOnOffAc( c, v ); break;

		case MX_EQU_TYPE     : S94_SetEquType( c, v ); break;

		case MX_TRANS_VAL    : S94_SetGenDev( c, v ); break;
		case MX_TRANS_REVSEND: S94_SetGenDev( c, v ); break;
		case MX_TRANS_CHRSEND: S94_SetGenDev( c, v ); break;

		case MX_EQ_LBL       : 
		case MX_EQ_MLBL      : 
		case MX_EQ_MHBL      : 
		case MX_EQ_HBL       : 

		case MX_EQF_LB       : 
		case MX_EQF_MLB      : 
		case MX_EQF_MHB      : 
		case MX_EQF_HB       : S94_SetGenDev( c, v ); S94_SetGenDev( c+4, v ); break;


		case MX_AUD_GAINL    : S94_SetGenDev( c, v ); S94_SetGenDev( c+1, v ); break;

		case MX_GM_VOL       : S94_SetGenDev( c, v ); break;
		case MX_GM_PAN       : S94_SetGenDev( c, v ); break;
		case MX_GMREV_SEND   : S94_SetGenDev( c, v ); break;
		case MX_GMCHR_SEND   : S94_SetGenDev( c, v ); break;

		case MX_AUDREV_SEND  : S94_SetGenDev( c, v ); break;
		case MX_ECH_LEV      : S94_SetGenDev( c, v ); break;
		case MX_ECH_TIM      : S94_SetGenDev( c, v ); break;
		case MX_ECH_FEED     : S94_SetGenDev( c, v ); break;
		case MX_SUR_VOL      : S94_SetGenDev( c, v ); break;
		case MX_SUR_DEL      : S94_SetGenDev( c, v ); break;

		case MX_AUDL_VOL     : S94_SetGenDev( c, v ); break;
		case MX_AUDR_VOL     : S94_SetGenDev( c, v ); break;

//		case MX_AUDL_PAN     : S94_SetGenDev( c, v ); break;
//		case MX_AUDR_PAN     : S94_SetGenDev( c, v ); break;

		case MX_REV_VOL      : S94_SetGenDev( c, v ); break;
		case MX_CHR_VOL      : S94_SetGenDev( c, v ); break;
		case MX_CHR_DEL      : S94_SetGenDev( c, v ); break;
		case MX_CHR_FEED     : S94_SetGenDev( c, v ); break;
		case MX_CHR_RATE     : S94_SetGenDev( c, v ); break;
		case MX_CHR_DEPTH    : S94_SetGenDev( c, v ); break;
		case MX_REV_TIME     : S94_SetGenDev( c, v ); break;
		case MX_REV_FEED     : S94_SetGenDev( c, v ); break;


		case MX_REC_MODE     : 
			{
				switch(v)
				{
					case 0 : S94_SetGenDev( c, 0x0 ); break;
					case 1 : S94_SetGenDev( c, 0x7f ); break;
					case 2 : S94_SetGenDev( c, 0x7e ); break;
					case 3 : S94_SetGenDev( c, 0x40 ); break;
			//		case 4 : HwSetGen( REC_POS, 0x7F ); break;
				}
			}

		case MX_REV_TYPE     : S94_SetGenDev( c, v ); break;
		case MX_CHR_TYPE     : S94_SetGenDev( c, v ); break;


		case MX_TRANS_ONOFF  : S94_SetGenDev( c, v ); break;
		case MX_TRANS_GMCH   : S94_SetGenDev( c, v ); break;


		case MX_W_VOLLEFT0   : Wav_VolLeft( 0, v ); Wav_VolRight( 0, v ); break;
		case MX_W_VOLLEFT1   : Wav_VolLeft( 1, v ); Wav_VolRight( 1, v ); break;
		case MX_W_VOLLEFT2   : Wav_VolLeft( 2, v ); Wav_VolRight( 2, v ); break;
		case MX_W_VOLLEFT3   : Wav_VolLeft( 3, v ); Wav_VolRight( 3, v ); break;
		case MX_W_VOLLEFT4   : Wav_VolLeft( 4, v ); Wav_VolRight( 4, v ); break;
		case MX_W_VOLLEFT5   : Wav_VolLeft( 5, v ); Wav_VolRight( 5, v ); break;
		case MX_W_VOLLEFT6   : Wav_VolLeft( 6, v ); Wav_VolRight( 6, v ); break;
		case MX_W_VOLLEFT7   : Wav_VolLeft( 7, v ); Wav_VolRight( 7, v ); break;
		case MX_W_VOLLEFT8   : Wav_VolLeft( 8, v ); Wav_VolRight( 8, v ); break;
 
		case MX_W_VOLAUXLEFT0: Wav_VolAuxLeft( 0, v ); Wav_VolAuxRight( 0, v ); break;
		case MX_W_VOLAUXLEFT1: Wav_VolAuxLeft( 1, v ); Wav_VolAuxRight( 1, v ); break;
		case MX_W_VOLAUXLEFT2: Wav_VolAuxLeft( 2, v ); Wav_VolAuxRight( 2, v ); break;
		case MX_W_VOLAUXLEFT3: Wav_VolAuxLeft( 3, v ); Wav_VolAuxRight( 3, v ); break;
		case MX_W_VOLAUXLEFT4: Wav_VolAuxLeft( 4, v ); Wav_VolAuxRight( 4, v ); break;
		case MX_W_VOLAUXLEFT5: Wav_VolAuxLeft( 5, v ); Wav_VolAuxRight( 5, v ); break;
		case MX_W_VOLAUXLEFT6: Wav_VolAuxLeft( 6, v ); Wav_VolAuxRight( 6, v ); break;
		case MX_W_VOLAUXLEFT7: Wav_VolAuxLeft( 7, v ); Wav_VolAuxRight( 7, v ); break;
		case MX_W_VOLAUXLEFT8: Wav_VolAuxLeft( 8, v ); Wav_VolAuxRight( 8, v ); break;

		case MX_W_PITCH      : 
		case MX_W_FILT_FC    : 
		case MX_W_FILT_Q     : S94_SetWordGen( c, 0, v ); break;

		case MX_VOI_VOL      : for( i=0;i<32;i++) Mod_Voi_Vol(i,v);      break;
		case MX_VOI_MAIN     : for( i=0;i<32;i++) Mod_Voi_Main (i,v,v);  break;
		case MX_VOI_PITCH    : for( i=0;i<32;i++) Mod_Voi_Pitch (i,v);   break;
		case MX_VOI_AUX      : for( i=0;i<32;i++) Mod_Voi_Aux (i,v,v);   break;
		case MX_VOI_FILT     : for( i=0;i<32;i++) Mod_Voi_Filter(i,v,v); break;

		case MX_AUDCHR_SEND  : S94_SetGenDev( c, v ); break;

		case MX_MASTER_VOL   : S94_SetGenDev( c, v ); break;

		case MX_W_PITCH0     : Wav_Pitch( 0, v );  break;
		case MX_W_PITCH1     : Wav_Pitch( 1, v );  break;
		case MX_W_PITCH2     : Wav_Pitch( 2, v );  break;
		case MX_W_PITCH3     : Wav_Pitch( 3, v );  break;
		case MX_W_PITCH4     : Wav_Pitch( 4, v );  break;
		case MX_W_PITCH5     : Wav_Pitch( 5, v );  break;
		case MX_W_PITCH6     : Wav_Pitch( 6, v );  break;
		case MX_W_PITCH7     : Wav_Pitch( 7, v );  break;

		case MX_W_FC0        : Wav_Filt_FC( 0, v ); break;
		case MX_W_FC1        : Wav_Filt_FC( 1, v ); break;
		case MX_W_FC2        : Wav_Filt_FC( 2, v ); break;
		case MX_W_FC3        : Wav_Filt_FC( 3, v ); break;
		case MX_W_FC4        : Wav_Filt_FC( 4, v ); break;
		case MX_W_FC5        : Wav_Filt_FC( 5, v ); break;
		case MX_W_FC6        : Wav_Filt_FC( 6, v ); break;
		case MX_W_FC7        : Wav_Filt_FC( 7, v ); break;

		case MX_W_Q0         : Wav_Filt_Q ( 0, v ); break;
		case MX_W_Q1         : Wav_Filt_Q ( 1, v ); break;
		case MX_W_Q2         : Wav_Filt_Q ( 2, v ); break;
		case MX_W_Q3         : Wav_Filt_Q ( 3, v ); break;
		case MX_W_Q4         : Wav_Filt_Q ( 4, v ); break;
		case MX_W_Q5         : Wav_Filt_Q ( 5, v ); break;
		case MX_W_Q6         : Wav_Filt_Q ( 6, v ); break;
		case MX_W_Q7         : Wav_Filt_Q ( 7, v ); break;

		case SB_MASTER 	: 
		case SB_WAV 	: 
		case SB_FM 	: 
		case SB_CD 	: 
		case SB_LINE  	: 
		case SB_MIC   	: 
			Set_SB_Mixer ( id, v );
			break;

	}
}
