/*
  Copyright (C) 1998, 1999 Gerd Rausch <gerd@alf.gun.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <asm/io.h>

#include <tcl.h>

typedef int BOOL;
typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long  DWORD;
typedef char *LPSTR;

#include "controls.h"
#include "sam9407.h"

#define NumberOf(vec) (sizeof(vec)/sizeof(*(vec)))

#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

static Tcl_HashTable optionsByName;
static Tcl_DString notifyCmd;

MIXRES MixRes[] = {
	{ "TRANS_VAL"    , MX_TRANS_VAL    , TRANS_VAL    , 0xff },
	{ "TRANS_REVSEND", MX_TRANS_REVSEND, TRANS_REVSEND, 0xff },
	{ "TRANS_CHRSEND", MX_TRANS_CHRSEND, TRANS_CHRSEND, 0xff },
	{ "EQ_LBL"       , MX_EQ_LBL       , EQ_LBL       , 0xff },
	{ "EQ_MLBL"      , MX_EQ_MLBL      , EQ_MLBL      , 0xff },
	{ "EQ_MHBL"      , MX_EQ_MHBL      , EQ_MHBL      , 0xff },
	{ "EQ_HBL"       , MX_EQ_HBL       , EQ_HBL       , 0xff },
	{ "EQF_LB"       , MX_EQF_LB       , EQF_LB       , 0xff },
	{ "EQF_MLB"      , MX_EQF_MLB      , EQF_MLB      , 0xff },
	{ "EQF_MHB"      , MX_EQF_MHB      , EQF_MHB      , 0xff },
	{ "EQF_HB"       , MX_EQF_HB       , EQF_HB       , 0xff },
	{ "AUD_GAINL"    , MX_AUD_GAINL    , AUD_GAINL    , 0xff },
//	{ "AUD_GAINR"    , MX_AUD_GAINR    , AUD_GAINR    , 0xff },
	{ "GMREV_SEND"   , MX_GMREV_SEND   , GMREV_SEND   , 0xff },
	{ "GMCHR_SEND"   , MX_GMCHR_SEND   , GMCHR_SEND   , 0xff },
	{ "AUDREV_SEND"  , MX_AUDREV_SEND  , AUDREV_SEND  , 0xff },
	{ "ECH_LEV"      , MX_ECH_LEV      , ECH_LEV      , 0xff },
	{ "ECH_TIM"      , MX_ECH_TIM      , ECH_TIM      , 0xff },
	{ "ECH_FEED"     , MX_ECH_FEED     , ECH_FEED     , 0xff },
	{ "SUR_VOL"      , MX_SUR_VOL      , SUR_VOL      , 0xff },
	{ "SUR_DEL"      , MX_SUR_DEL      , SUR_DEL      , 0xff },
	{ "SUR_INP"      , MX_SUR_INP      , SUR_INP      , 0xff },
	{ "AUDL_VOL"     , MX_AUDL_VOL     , AUDL_VOL     , 0xff },
	{ "AUDR_VOL"     , MX_AUDR_VOL     , AUDR_VOL     , 0xff },
//	{ "AUDL_PAN"     , MX_AUDL_PAN     , AUDL_PAN     , 0xff },
//	{ "AUDR_PAN"     , MX_AUDR_PAN     , AUDR_PAN     , 0xff },
	{ "GM_VOL"       , MX_GM_VOL       , GM_VOL       , 0xff },
	{ "GM_PAN"       , MX_GM_PAN       , GM_PAN       , 0xff },
	{ "REV_VOL"      , MX_REV_VOL      , REV_VOL      , 0xff },
	{ "CHR_VOL"      , MX_CHR_VOL      , CHR_VOL      , 0xff },
	{ "CHR_DEL"      , MX_CHR_DEL      , CHR_DEL      , 0xff },
	{ "CHR_FEED"     , MX_CHR_FEED     , CHR_FEED     , 0xff },
	{ "CHR_RATE"     , MX_CHR_RATE     , CHR_RATE     , 0xff },
	{ "CHR_DEPTH"    , MX_CHR_DEPTH    , CHR_DEPTH    , 0xff },
	{ "REV_TIME"     , MX_REV_TIME     , REV_TIME     , 0xff },
	{ "REV_FEED"     , MX_REV_FEED     , REV_FEED     , 0xff },
	{ "WAVE_ASS"     , MX_WAVE_ASS     , WAVE_ASS     , 0xff },
	{ "MOD_ASS"      , MX_MOD_ASS      , MOD_ASS      , 0xff },
	{ "GM_POST"      , MX_GM_POST      , GM_POST      , 0xff },
	{ "WAVE_POST"    , MX_WAVE_POST    , WAVE_POST    , 0xff },
	{ "MOD_POST"     , MX_MOD_POST     , MOD_POST     , 0xff },
	{ "AUDECH_POST"  , MX_AUDECH_POST  , AUDECH_POST  , 0xff },
	{ "EFF_POST"     , MX_EFF_POST     , EFF_POST     , 0xff },
	{ "ECH_ONOFF"    , MX_ECH_ONOFF    , ECH_ONOFF    , 0xff },
	{ "REV_ONOFF"    , MX_REV_ONOFF    , REV_ONOFF    , 0xff },
	{ "CHR_ONOFF"    , MX_CHR_ONOFF    , CHR_ONOFF    , 0xff },
	{ "SUR_ONOFF"    , MX_SUR_ONOFF    , SUR_ONOFF    , 0xff },
	{ "AUD_ONOFF"    , MX_AUD_ONOFF    , AUD_ONOFF    , 0xff },
	{ "REC_MODE"     , MX_REC_MODE     , REC_MODE     , 0xff },
	{ "AUD_SEL"      , MX_AUD_SEL      , AUD_SEL      , 0xff },
	{ "SUR_24"       , MX_SUR_24       , SUR_24       , 0xff },
	{ "REV_TYPE"     , MX_REV_TYPE     , REV_TYPE     , 0xff },
	{ "CHR_TYPE"     , MX_CHR_TYPE     , CHR_TYPE     , 0xff },
	{ "EQU_TYPE"     , MX_EQU_TYPE     , EQU_TYPE     , 0xff },
	{ "TRANS_ONOFF"  , MX_TRANS_ONOFF  , TRANS_ONOFF  , 0xff },
	{ "TRANS_GMCH"   , MX_TRANS_GMCH   , TRANS_GMCH   , 0xff },
	{ "W_PITCH"      , MX_W_PITCH      , W_PITCH      , 0xff },
	{ "W_VOLLEFT0"   , MX_W_VOLLEFT0   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT1"   , MX_W_VOLLEFT1   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT2"   , MX_W_VOLLEFT2   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT3"   , MX_W_VOLLEFT3   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT4"   , MX_W_VOLLEFT4   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT5"   , MX_W_VOLLEFT5   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT6"   , MX_W_VOLLEFT6   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT7"   , MX_W_VOLLEFT7   , W_VOLLEFT    , 0xff },
	{ "W_VOLLEFT8"   , MX_W_VOLLEFT8   , W_VOLLEFT    , 0xff },
	{ "W_VOLAUXLEFT0", MX_W_VOLAUXLEFT0, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT1", MX_W_VOLAUXLEFT1, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT2", MX_W_VOLAUXLEFT2, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT3", MX_W_VOLAUXLEFT3, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT4", MX_W_VOLAUXLEFT4, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT5", MX_W_VOLAUXLEFT5, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT6", MX_W_VOLAUXLEFT6, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT7", MX_W_VOLAUXLEFT7, W_VOLAUXLEFT , 0xff },
	{ "W_VOLAUXLEFT8", MX_W_VOLAUXLEFT8, W_VOLAUXLEFT , 0xff },
	{ "W_FILT_FC"    , MX_W_FILT_FC    , W_FILT_FC    , 0xff },
	{ "W_FILT_Q"     , MX_W_FILT_Q     , W_FILT_Q     , 0xff },
	{ "VOI_VOL"      , MX_VOI_VOL      , VOI_VOL      , 0xff },
	{ "VOI_MAIN"     , MX_VOI_MAIN     , VOI_MAIN     , 0xff },
	{ "VOI_PITCH"    , MX_VOI_PITCH    , VOI_PITCH    , 0xff },
	{ "VOI_AUX"      , MX_VOI_AUX      , VOI_AUX      , 0xff },
	{ "VOI_FILT"     , MX_VOI_FILT     , VOI_FILT     , 0xff },
	{ "ECH_SILENT"   , MX_ECH_SILENT   , ECH_ONOFF    , 0xff },
	{ "EFF_SILENT"   , MX_EFF_SILENT   , REV_ONOFF    , 0xff },
	{ "EQU_SILENT"   , MX_EQU_SILENT   , EQU_TYPE     , 0xff },
	{ "AUDCHR_SEND"  , MX_AUDCHR_SEND  , AUDCHR_SEND  , 0xff },
	{ "MASTER_VOL"   , MX_MASTER_VOL   , MASTER_VOL   , 0xff },
	{ "W_PITCH0"     , MX_W_PITCH0     , W_PITCH      , 0xff },
	{ "W_PITCH1"     , MX_W_PITCH1     , W_PITCH      , 0xff },
	{ "W_PITCH2"     , MX_W_PITCH2     , W_PITCH      , 0xff },
	{ "W_PITCH3"     , MX_W_PITCH3     , W_PITCH      , 0xff },
	{ "W_PITCH4"     , MX_W_PITCH4     , W_PITCH      , 0xff },
	{ "W_PITCH5"     , MX_W_PITCH5     , W_PITCH      , 0xff },
	{ "W_PITCH6"     , MX_W_PITCH6     , W_PITCH      , 0xff },
	{ "W_PITCH7"     , MX_W_PITCH7     , W_PITCH      , 0xff },
	{ "W_FC0"        , MX_W_FC0        , W_FILT_FC    , 0xff },
	{ "W_FC1"        , MX_W_FC1        , W_FILT_FC    , 0xff },
	{ "W_FC2"        , MX_W_FC2        , W_FILT_FC    , 0xff },
	{ "W_FC3"        , MX_W_FC3        , W_FILT_FC    , 0xff },
	{ "W_FC4"        , MX_W_FC4        , W_FILT_FC    , 0xff },
	{ "W_FC5"        , MX_W_FC5        , W_FILT_FC    , 0xff },
	{ "W_FC6"        , MX_W_FC6        , W_FILT_FC    , 0xff },
	{ "W_FC7"        , MX_W_FC7        , W_FILT_FC    , 0xff },

	{ "W_Q0"         , MX_W_Q0         , W_FILT_Q     , 0xff },
	{ "W_Q1"         , MX_W_Q1         , W_FILT_Q     , 0xff },
	{ "W_Q2"         , MX_W_Q2         , W_FILT_Q     , 0xff },
	{ "W_Q3"         , MX_W_Q3         , W_FILT_Q     , 0xff },
	{ "W_Q4"         , MX_W_Q4         , W_FILT_Q     , 0xff },
	{ "W_Q5"         , MX_W_Q5         , W_FILT_Q     , 0xff },
	{ "W_Q6"         , MX_W_Q6         , W_FILT_Q     , 0xff },
	{ "W_Q7"         , MX_W_Q7         , W_FILT_Q     , 0xff }, 

	{ "SB_MASTER"    , SB_MASTER       , SB_MASTER    , 0xff }, 
	{ "SB_WAV"       , SB_WAV          , SB_WAV       , 0xff }, 
	{ "SB_FM"        , SB_FM           , SB_FM        , 0xff }, 
	{ "SB_CD"        , SB_CD           , SB_CD        , 0xff }, 
	{ "SB_LINE"      , SB_LINE         , SB_LINE      , 0xff }, 
	{ "SB_MIC"       , SB_MIC          , SB_MIC       , 0xff }, 
	{ "End Volume"   , 0xffff,0, 0xff },
};

int id,val;

static int parseOptions( Tcl_Interp *interp, int argc, char **argv )
{
  Tcl_HashEntry *entry;
  int n=-1, i;

  for(i=2; i<argc;i++)
  {
	printf( "%d : %s\n", i, argv[i] );
	if(!(entry=Tcl_FindHashEntry(&optionsByName, argv[i])))
	{
	  Tcl_AppendResult(interp, "mixer: unknown mixer option: \"", argv[i], "\"", NULL);
	  return -1;
	}

	i++;
	id  = n = (int)Tcl_GetHashValue(entry);
	val = strtol( argv[i], NULL, 0 );
	
	//break;
  }
  return 1;
}

static int mixerCmd(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
  Tcl_HashEntry *entry;
  Tcl_DString dstr;
  char buf[32];
  int n=-1, i=0;

  printf( "%s\n", argv[1] );
  
  if(argc>1 && strcmp(argv[1], "get")==0)
  {
	//if((n=parseOptions( interp, argc, argv )<0))
	//  return TCL_ERROR;

	Tcl_DStringInit( &dstr );
	sprintf( buf, "%d", 0xff );
	Tcl_DStringAppendElement( &dstr, buf );

	Tcl_DStringResult(interp, &dstr );
	Tcl_DStringFree( &dstr );
  }
  else if(argc>1 && strcmp(argv[1], "set")==0)
  {
	if((n=parseOptions( interp, argc, argv )<0))
	  return TCL_ERROR;

        printf( "%d %d\n", id, val );
	MixerMessage( id, val );

	Tcl_ResetResult(interp);
  }
  else if(argc>1 && strcmp(argv[1], "info")==0)
  {
        Tcl_DStringInit( &dstr );
	
        for(i=0; i<sizeof(MixRes) / sizeof(MixRes[0]); i++)
            Tcl_DStringAppendElement( &dstr, MixRes[i].name );
	    
	Tcl_DStringResult(interp, &dstr );
  }
  else if(argc==3 && strcmp(argv[1], "notify")==0)
  {
        //Tcl_DStringSetLength( &notifyCmd, 0 );
        //Tcl_DStringAppend( &notifyCmdd, argv[2], -1 );
  }
  return TCL_OK;
}

int Sam_Init(Tcl_Interp *interp)
{
  int i, newEntry;
  Tcl_HashEntry *entry;

  Tcl_InitHashTable(&optionsByName, TCL_STRING_KEYS);

  iopl(3);
  
  for(i=0; i<sizeof(MixRes) / sizeof(MixRes[0]); i++) 
  {
	entry=Tcl_CreateHashEntry(&optionsByName, MixRes[i].name, &newEntry);
	Tcl_SetHashValue(entry, (ClientData)MixRes[i].id);
  }

  Tcl_CreateCommand(interp, "mixer", (Tcl_CmdProc *)mixerCmd, NULL, NULL);

  return TCL_OK;
}
